@@ -34,13 +34,22 @@ import pickle
 import shelve
 import sys
 import time
+
 from ast import literal_eval
 from gettext import gettext as _
 from os.path import exists
 
-import _thread
 from pynicotine.logfacility import log
 
+if sys.platform == "win32":
+    # Use semidbm for faster shelves on Windows
+
+    def shelve_open_semidbm(filename, flag='c', protocol=None, writeback=False):
+        import semidbm
+        return shelve.Shelf(semidbm.open(filename, flag), protocol, writeback)
+
+    shelve.open = shelve_open_semidbm
+
 
 class RestrictedUnpickler(pickle.Unpickler):
     """
@@ -62,7 +71,6 @@ class Config:
     setConfig(config_info_dict) - sets configuration information
     writeConfiguration - writes configuration information to file
     writeDownloadQueue - writes download queue to file
-    writeConfig - calls writeConfiguration followed by writeDownloadQueue
 
     The actual configuration information is stored as a two-level dictionary.
     First-level keys are config sections, second-level keys are config
@@ -71,13 +79,16 @@ class Config:
 
     def __init__(self, filename, data_dir):
 
-        self.config_lock = _thread.allocate_lock()
-        self.config_lock.acquire()
         self.frame = None
         self.filename = filename
         self.data_dir = data_dir
         self.parser = configparser.RawConfigParser()
-        self.parser.read([self.filename])
+
+        try:
+            self.parser.read([self.filename], encoding="utf-8")
+        except UnicodeDecodeError:
+            self.convertConfig()
+            self.parser.read([self.filename], encoding="utf-8")
 
         LOGDIR = os.path.join(data_dir, "logs")
 
@@ -86,8 +97,8 @@ class Config:
                 "server": ('server.slsknet.org', 2242),
                 "login": '',
                 "passw": '',
-                "firewalled": 1,
-                "ctcpmsgs": 0,
+                "firewalled": False,
+                "ctcpmsgs": False,
                 "autosearch": [],
                 "autoreply": "",
                 "portrange": (2234, 2239),
@@ -99,53 +110,51 @@ class Config:
                 "ipblocklist": {"72.172.88.*": "MediaDefender Bots"},
                 "autojoin": ["nicotine"],
                 "autoaway": 15,
-                "private_chatrooms": 0
+                "private_chatrooms": False
             },
 
             "transfers": {
                 "incompletedir": os.path.join(data_dir, 'incompletefiles'),
                 "downloaddir": os.path.join(os.path.expanduser("~"), 'nicotine-downloads'),
                 "uploaddir": os.path.join(os.path.expanduser("~"), 'nicotine-uploads'),
-                "sharedownloaddir": 0,
+                "sharedownloaddir": False,
                 "shared": [],
                 "buddyshared": [],
                 "uploadbandwidth": 10,
-                "uselimit": 0,
+                "uselimit": False,
                 "uploadlimit": 150,
                 "downloadlimit": 0,
-                "preferfriends": 0,
-                "useupslots": 0,
+                "preferfriends": False,
+                "useupslots": False,
                 "uploadslots": 2,
-                "shownotification": 0,
-                "shownotificationperfolder": 0,
                 "afterfinish": "",
                 "afterfolder": "",
-                "lock": 1,
-                "reverseorder": 0,
-                "prioritize": 0,
-                "fifoqueue": 0,
-                "usecustomban": 0,
-                "limitby": 1,
+                "lock": True,
+                "reverseorder": False,
+                "prioritize": False,
+                "fifoqueue": False,
+                "usecustomban": False,
+                "limitby": True,
                 "customban": "Banned, don't bother retrying",
                 "queuelimit": 10000,
                 "filelimit": 1000,
-                "friendsonly": 0,
-                "friendsnolimits": 0,
-                "enablebuddyshares": 0,
-                "enabletransferbuttons": 1,
+                "friendsonly": False,
+                "friendsnolimits": False,
+                "enablebuddyshares": False,
+                "enabletransferbuttons": True,
                 "groupdownloads": True,
                 "groupuploads": True,
-                "geoblock": 0,
-                "geopanic": 0,
+                "geoblock": False,
+                "geopanic": False,
                 "geoblockcc": [""],
-                "remotedownloads": 1,
+                "remotedownloads": True,
                 "uploadallowed": 2,
-                "autoclear_downloads": 0,
-                "autoclear_uploads": 0,
+                "autoclear_downloads": False,
+                "autoclear_uploads": False,
                 "downloads": [],
                 "sharedfiles": {},
                 "sharedfilesstreams": {},
-                "uploadsinsubdirs": 1,
+                "uploadsinsubdirs": True,
                 "wordindex": {},
                 "fileindex": {},
                 "sharedmtimes": {},
@@ -155,7 +164,7 @@ class Config:
                 "bfileindex": {},
                 "bsharedmtimes": {},
                 "rescanonstartup": 0,
-                "enablefilters": 1,
+                "enablefilters": True,
                 "downloadregexp": "",
                 "downloadfilters": [
                     ["desktop.ini", 1],
@@ -204,25 +213,25 @@ class Config:
             "logging": {
                 "debug": False,
                 "debugmodes": [0, 1],
-                "logcollapsed": 0,
+                "logcollapsed": False,
                 "logsdir": LOGDIR,
                 "rooms_timestamp": "%H:%M:%S",
                 "private_timestamp": "%Y-%m-%d %H:%M:%S",
                 "log_timestamp": "%Y-%m-%d %H:%M:%S",
-                "timestamps": 1,
-                "privatechat": 0,
-                "chatrooms": 0,
-                "transfers": 0,
+                "timestamps": True,
+                "privatechat": False,
+                "chatrooms": False,
+                "transfers": False,
                 "roomlogsdir": os.path.join(LOGDIR, "rooms"),
                 "privatelogsdir": os.path.join(LOGDIR, "private"),
-                "readroomlogs": 1,
+                "readroomlogs": True,
                 "readroomlines": 15,
                 "readprivatelines": 15,
                 "rooms": []
             },
 
             "privatechat": {
-                "store": 0,
+                "store": False,
                 "users": []
             },
 
@@ -230,7 +239,7 @@ class Config:
                 "userbrowse": [1, 1, 1, 1],
                 "userbrowse_widths": [600, 100, 70, 0],
                 "userlist": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
-                "userlist_widths": [0, 25, 120, 0, 0, 0, 0, 0, 160, 0],
+                "userlist_widths": [0, 25, 180, 0, 0, 0, 0, 0, 160, 0],
                 "chatrooms": {},
                 "chatrooms_widths": {},
                 "download_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
@@ -238,17 +247,17 @@ class Config:
                 "upload_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                 "upload_widths": [200, 250, 250, 140, 50, 70, 170, 90, 140, 0],
                 "filesearch_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
-                "filesearch_widths": [50, 200, 25, 20, 90, 50, 400, 400, 100, 50, 0],
+                "filesearch_widths": [50, 200, 25, 50, 90, 90, 400, 400, 100, 100, 0],
                 "hideflags": False
             },
 
             "searches": {
                 "expand_searches": True,
-                "group_searches": True,
+                "group_searches": 1,
                 "maxresults": 50,
-                "re_filter": 0,
+                "re_filter": False,
                 "history": [],
-                "enablefilters": 0,
+                "enablefilters": False,
                 "defilter": ["", "", "", "", 0, ""],
                 "filtercc": [],
                 "reopen_tabs": False,
@@ -256,11 +265,12 @@ class Config:
                 "filterout": [],
                 "filtersize": [],
                 "filterbr": [],
-                "distrib_timer": 0,
+                "distrib_timer": False,
                 "distrib_ignore": 60,
-                "search_results": 1,
+                "search_results": True,
                 "max_displayed_results": 1000,
                 "max_stored_results": 1500,
+                "min_search_chars": 3,
                 "remove_special_chars": True
             },
 
@@ -274,30 +284,28 @@ class Config:
                 "useronline": "BLACK",
                 "useraway": "ORANGE",
                 "useroffline": "#aa0000",
-                "usernamehotspots": 1,
+                "usernamehotspots": True,
                 "usernamestyle": "bold",
                 "textbg": "",
                 "search": "",
                 "searchq": "GREY",
                 "inputcolor": "",
-                "spellcheck": 1,
+                "spellcheck": True,
                 "exitdialog": 1,
-                "notexists": 1,
-                "tab_colors": 0,
+                "notexists": True,
                 "tab_default": "",
                 "tab_hilite": "red",
                 "tab_changed": "#0000ff",
-                "tab_select_previous": 1,
-                "tab_reorderable": 1,
+                "tab_select_previous": True,
+                "tab_reorderable": True,
                 "tabmain": "Top",
                 "tabrooms": "Top",
                 "tabprivate": "Top",
                 "tabinfo": "Top",
                 "tabbrowse": "Top",
                 "tabsearch": "Top",
-                "tab_icons": 1,
-                "tab_status_icons": 1,
-                "chat_hidebuttons": 0,
+                "tab_status_icons": True,
+                "chat_hidebuttons": False,
                 "labelmain": 0,
                 "labelrooms": 0,
                 "labelprivate": 0,
@@ -306,8 +314,8 @@ class Config:
                 "labelsearch": 0,
                 "decimalsep": ",",
                 "chatfont": "",
-                "roomlistcollapsed": 0,
-                "tabclosers": 1,
+                "roomlistcollapsed": False,
+                "tabclosers": True,
                 "searchfont": "",
                 "listfont": "",
                 "browserfont": "",
@@ -334,14 +342,11 @@ class Config:
                     "interests",
                     "userlist"
                 ],
-                "searchoffline": "#aa0000",
-                "showaway": 0,
+                "showaway": False,
                 "buddylistinchatrooms": 0,
-                "trayicon": 1,
-                "soundenabled": 1,
-                "soundtheme": "",
+                "trayicon": True,
                 "filemanager": "xdg-open $",
-                "speechenabled": 0,
+                "speechenabled": False,
                 "speechprivate": "%(user)s told you.. %(message)s",
                 "speechrooms": "In %(room)s, %(user)s said %(message)s",
                 "speechcommand": "flite -t $",
@@ -354,14 +359,13 @@ class Config:
             },
 
             "private_rooms": {
-                "rooms": {},
-                "enabled": 0
+                "rooms": {}
             },
 
             "urls": {
-                "urlcatching": 1,
+                "urlcatching": True,
                 "protocols": {},
-                "humanizeurls": 1
+                "humanizeurls": True
             },
 
             "interests": {
@@ -369,12 +373,6 @@ class Config:
                 "dislikes": []
             },
 
-            "ticker": {
-                "default": "",
-                "rooms": {},
-                "hide": 0
-            },
-
             "players": {
                 "default": "xdg-open $",
                 "npothercommand": "",
@@ -383,8 +381,20 @@ class Config:
                 "npformat": ""
             },
 
+            "notifications": {
+                "notification_window_title": True,
+                "notification_tab_colors": False,
+                "notification_tab_icons": True,
+                "notification_popup_sound": True,
+                "notification_popup_file": True,
+                "notification_popup_folder": True,
+                "notification_popup_private_message": True,
+                "notification_popup_chatroom": False,
+                "notification_popup_chatroom_mention": True
+            },
+
             "plugins": {
-                "enable": 1,
+                "enable": True,
                 "enabled": []
             }
         }
@@ -394,12 +404,12 @@ class Config:
             self.sections['ui']['filemanager'] = 'explorer $'
 
         self.defaults = {}
-        for key, value in list(self.sections.items()):
+        for key, value in self.sections.items():
             if type(value) is dict:
                 if key not in self.defaults:
                     self.defaults[key] = {}
 
-                for key2, value2 in list(value.items()):
+                for key2, value2 in value.items():
                     self.defaults[key][key2] = value2
             else:
                 self.defaults[key] = value
@@ -411,21 +421,42 @@ class Config:
         except Exception:
             self.aliases = {}
 
-        self.config_lock.release()
+    def convertConfig(self):
+        """ Converts the config to utf-8.
+        Mainly for upgrading Windows build. (22 July, 2020) """
+
+        try:
+            from chardet import detect
+        except ImportError:
+            return
+
+        os.rename(self.filename, self.filename + ".conv")
+
+        with open(self.filename + ".conv", 'rb') as f:
+            rawdata = f.read()
+
+        from_encoding = detect(rawdata)['encoding']
+
+        with open(self.filename + ".conv", 'r', encoding=from_encoding) as fr:
+            with open(self.filename, 'w', encoding="utf-8") as fw:
+                for line in fr:
+                    fw.write(line[:-1] + '\r\n')
+
+        os.remove(self.filename + ".conv")
 
     def needConfig(self):
 
         errorlevel = 0
 
         try:
-            for i in list(self.sections.keys()):
-                for j in list(self.sections[i].keys()):
+            for i in self.sections:
+                for j in self.sections[i]:
 
                     if type(self.sections[i][j]) not in [type(None), type("")]:
                         continue
 
                     if self.sections[i][j] is None or self.sections[i][j] == '' \
-                       and i not in ("userinfo", "ui", "ticker", "players") \
+                       and i not in ("userinfo", "ui", "players") \
                        and j not in ("incompletedir", "autoreply", 'afterfinish', 'afterfolder', 'geoblockcc', 'downloadregexp'):
 
                         # Repair options set to None with defaults
@@ -466,8 +497,6 @@ class Config:
 
     def readConfig(self):
 
-        self.config_lock.acquire()
-
         self.sections['transfers']['downloads'] = []
 
         if exists(os.path.join(self.data_dir, 'transfers.pickle')):
@@ -479,7 +508,7 @@ class Config:
             else:
                 try:
                     self.sections['transfers']['downloads'] = RestrictedUnpickler(handle).load()
-                except (IOError, EOFError, ValueError) as inst:
+                except Exception as inst:
                     log.addwarning(_("Something went wrong while reading your transfer list: %(error)s") % {'error': str(inst)})
             try:
                 handle.close()
@@ -541,6 +570,23 @@ class Config:
         # Remove auto-retry failed downloads-option, this is now default behavior
         self.removeOldOption("transfers", "autoretry_downloads")
 
+        # Remove old notification/sound settings
+        self.removeOldOption("transfers", "shownotification")
+        self.removeOldOption("transfers", "shownotificationperfolder")
+        self.removeOldOption("ui", "soundenabled")
+        self.removeOldOption("ui", "soundtheme")
+        self.removeOldOption("ui", "tab_colors")
+        self.removeOldOption("ui", "tab_icons")
+
+        # Remove dropped offline user text color in search results
+        self.removeOldOption("ui", "searchoffline")
+
+        # Seems to be superseded by ("server", "private_chatrooms")
+        self.removeOldOption("private_rooms", "enabled")
+
+        # Remove everything ticker-related, no longer necessary after the introduction of room walls
+        self.removeOldSection("ticker")
+
         # Checking for unknown section/options
         unknown1 = [
             'login', 'passw', 'enc', 'downloaddir', 'uploaddir', 'customban',
@@ -557,7 +603,7 @@ class Config:
                 "showaway", "usernamehotspots", "exitdialog",
                 "tab_icons", "spellcheck", "modes_order", "modes_visible",
                 "chat_hidebuttons", "tab_status_icons", "notexists",
-                "soundenabled", "speechenabled", "enablefilters", "width",
+                "speechenabled", "enablefilters", "width",
                 "height", "xposition", "yposition", "labelmain", "labelrooms",
                 "labelprivate", "labelinfo", "labelbrowse", "labelsearch"
             ],
@@ -675,8 +721,6 @@ class Config:
         # Setting the port range in numerical order
         self.sections["server"]["portrange"] = (min(self.sections["server"]["portrange"]), max(self.sections["server"]["portrange"]))
 
-        self.config_lock.release()
-
     def removeOldOption(self, section, option):
         if section in self.parser.sections():
             if option in self.parser.options(section):
@@ -776,12 +820,8 @@ class Config:
             return None
         return sharedfiles, bsharedfiles, sharedfilesstreams, bsharedfilesstreams, wordindex, bwordindex, fileindex, bfileindex, sharedmtimes, bsharedmtimes
 
-    def writeConfig(self):
-        self.writeConfiguration()
-        self.writeDownloadQueue()
-
     def writeDownloadQueue(self):
-        self.config_lock.acquire()
+
         realfile = os.path.join(self.data_dir, 'transfers.pickle')
         tmpfile = realfile + '.tmp'
         backupfile = realfile + ' .backup'
@@ -796,7 +836,7 @@ class Config:
                 try:
                     # Please let it be atomic...
                     os.rename(tmpfile, realfile)
-                except Exception as inst:  # noqa: F841
+                except Exception:
                     # ...ugh. Okay, how about...
                     try:
                         os.unlink(backupfile)
@@ -811,22 +851,19 @@ class Config:
                 handle.close()
             except Exception:
                 pass
-        self.config_lock.release()
 
     def writeConfiguration(self):
 
-        self.config_lock.acquire()
-
         external_sections = [
             "sharedfiles", "sharedfilesstreams", "wordindex", "fileindex",
             "sharedmtimes", "bsharedfiles", "bsharedfilesstreams",
             "bwordindex", "bfileindex", "bsharedmtimes", "downloads"
         ]
 
-        for i in list(self.sections.keys()):
+        for i in self.sections:
             if not self.parser.has_section(i):
                 self.parser.add_section(i)
-            for j in list(self.sections[i].keys()):
+            for j in self.sections[i]:
                 if j not in external_sections:
                     self.parser.set(i, j, self.sections[i][j])
                 else:
@@ -842,17 +879,15 @@ class Config:
         oldumask = os.umask(0o077)
 
         try:
-            f = open(self.filename + ".new", "w")
+            f = open(self.filename + ".new", "w", encoding="utf-8")
         except IOError as e:
             log.addwarning(_("Can't save config file, I/O error: %s") % e)
-            self.config_lock.release()
             return
         else:
             try:
                 self.parser.write(f)
             except IOError as e:
                 log.addwarning(_("Can't save config file, I/O error: %s") % e)
-                self.config_lock.release()
                 return
             else:
                 f.close()
@@ -871,7 +906,7 @@ class Config:
                 try:
                     if os.path.exists(self.filename + ".old"):
                         os.remove(self.filename + ".old")
-                except OSError as error:  # noqa: F841
+                except OSError:
                     log.addwarning(_("Can't remove %s" % self.filename + ".old"))
                 try:
                     os.rename(self.filename, self.filename + ".old")
@@ -885,12 +920,8 @@ class Config:
         except OSError as error:
             log.addwarning(_("Can't rename config file, error: %s") % error)
 
-        self.config_lock.release()
-
     def writeConfigBackup(self, filename=None):
 
-        self.config_lock.acquire()
-
         if filename is None:
             filename = "%s backup %s.tar.bz2" % (self.filename, time.strftime("%Y-%m-%d %H:%M:%S"))
         else:
@@ -910,9 +941,8 @@ class Config:
             tar.close()
         except Exception as e:
             print(e)
-            self.config_lock.release()
             return (1, "Cannot write backup archive: %s" % e)
-        self.config_lock.release()
+
         return (0, filename)
 
     def setBuddyShares(self, files, streams, wordindex, fileindex, mtimes):
@@ -925,9 +955,7 @@ class Config:
             (fileindex, "bfileindex", "buddyfileindex.db")
         ]
 
-        self.config_lock.acquire()
-        self._storeObjects(storable_objects)
-        self.config_lock.release()
+        self.storeObjects(storable_objects)
 
     def setShares(self, files, streams, wordindex, fileindex, mtimes):
 
@@ -939,51 +967,31 @@ class Config:
             (fileindex, "fileindex", "fileindex.db")
         ]
 
-        self.config_lock.acquire()
-        self._storeObjects(storable_objects)
-        self.config_lock.release()
+        self.storeObjects(storable_objects)
 
-    def _storeObjects(self, storable_objects):
+    def storeObjects(self, storable_objects):
 
         for (source, destination, filename) in storable_objects:
             try:
                 self.sections["transfers"][destination].close()
                 self.sections["transfers"][destination] = shelve.open(os.path.join(self.data_dir, filename), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
-                for (key, value) in source.items():
-                    self.sections["transfers"][destination][key] = value
+                if "fileindex" in destination:
+                    index = 0
+
+                    for value in source:
+                        # The file index db is a list
+                        self.sections["transfers"][destination][str(index)] = value
+                        index += 1
+                else:
+                    for key in source:
+                        self.sections["transfers"][destination][key] = source[key]
             except Exception as e:
                 log.addwarning(_("Can't save %s: %s") % (filename, e))
                 return
 
-    def writeShares(self):
-
-        self.config_lock.acquire()
-
-        self.sections["transfers"]["sharedfiles"].sync()
-        self.sections["transfers"]["sharedfilesstreams"].sync()
-        self.sections["transfers"]["wordindex"].sync()
-        self.sections["transfers"]["fileindex"].sync()
-        self.sections["transfers"]["sharedmtimes"].sync()
-
-        self.sections["transfers"]["bsharedfiles"].sync()
-        self.sections["transfers"]["bsharedfilesstreams"].sync()
-        self.sections["transfers"]["bwordindex"].sync()
-        self.sections["transfers"]["bfileindex"].sync()
-        self.sections["transfers"]["bsharedmtimes"].sync()
-
-        self.config_lock.release()
-
-    def pushHistory(self, history, text, max):
-        if text in history:
-            history.remove(text)
-        elif len(history) >= max:
-            del history[-1]
-        history.insert(0, text)
-        self.writeConfig()
-
     def writeAliases(self):
-        self.config_lock.acquire()
+
         try:
             f = open(self.filename + ".alias", "wb")
         except Exception as e:
@@ -999,7 +1007,6 @@ class Config:
                 f.close()
             except Exception:
                 pass
-        self.config_lock.release()
 
     def AddAlias(self, rest):
         if rest:
