@@ -36,8 +36,11 @@ from gi.repository import Pango as pango
 
 from pynicotine import slskmessages
 from pynicotine.gtkgui import nowplaying
-from pynicotine.gtkgui.entrydialog import input_box
+from pynicotine.gtkgui.dialogs import EntryDialog
+from pynicotine.gtkgui.roomwall import RoomWall
+from pynicotine.gtkgui.roomwall import Tickers
 from pynicotine.gtkgui.utils import AppendLine
+from pynicotine.gtkgui.utils import HideColumns
 from pynicotine.gtkgui.utils import Humanize
 from pynicotine.gtkgui.utils import HumanSpeed
 from pynicotine.gtkgui.utils import IconNotebook
@@ -47,10 +50,10 @@ from pynicotine.gtkgui.utils import PressHeader
 from pynicotine.gtkgui.utils import ScrollBottom
 from pynicotine.gtkgui.utils import WriteLog
 from pynicotine.gtkgui.utils import expand_alias
-from pynicotine.gtkgui.utils import fixpath
 from pynicotine.gtkgui.utils import is_alias
 from pynicotine.gtkgui.utils import showCountryTooltip
 from pynicotine.logfacility import log
+from pynicotine.utils import CleanFile
 from pynicotine.utils import cmp
 from pynicotine.utils import debug
 
@@ -77,8 +80,7 @@ class RoomsControl:
         "/alias ", "/unalias ", "/whois ", "/browse ", "/ip ", "/pm ", "/msg ", "/search ",
         "/usearch ", "/rsearch ", "/bsearch ", "/join ", "/leave ", "/add ", "/buddy ", "/rem ",
         "/unbuddy ", "/ban ", "/ignore ", "/ignoreip ", "/unban ", "/unignore ", "/clear ",
-        "/part ", "/quit ", "/exit ", "/rescan ", "/tick ", "/info ", "/attach ", "/detach ",
-        "/toggle", "/tickers"
+        "/part ", "/quit ", "/exit ", "/rescan ", "/tick ", "/info ", "/toggle", "/tickers"
     }
 
     def __init__(self, ChatNotebook):
@@ -120,15 +122,13 @@ class RoomsControl:
 
         self.popup_room = None
         self.popup_menu = PopupMenu(self.frame).setup(
-            ("#" + _("Join room"), self.OnPopupJoin),
-            ("#" + _("Leave room"), self.OnPopupLeave),
-            ("", None),
-            ("#" + _("Enable Private Rooms"), self.OnEnablePrivateRooms),
-            ("#" + _("Disable Private Rooms"), self.OnDisablePrivateRooms),
+            ("#" + _("Join Room"), self.OnPopupJoin),
+            ("#" + _("Leave Room"), self.OnPopupLeave),
+            ("#" + _("Create Room"), self.OnPopupCreatePublicRoom),
             ("", None),
             ("#" + _("Create Private Room"), self.OnPopupCreatePrivateRoom),
             ("#" + _("Disown Private Room"), self.OnPopupPrivateRoomDisown),
-            ("#" + _("Cancel room membership"), self.OnPopupPrivateRoomDismember),
+            ("#" + _("Cancel Room Membership"), self.OnPopupPrivateRoomDismember),
             ("", None),
             ("#" + _("Join Public Room"), self.OnJoinPublicRoom),
             ("", None),
@@ -136,11 +136,7 @@ class RoomsControl:
         )
 
         items = self.popup_menu.get_children()
-        self.Menu_Join, self.Menu_Leave, self.Menu_Empty1, self.Menu_PrivateRoom_Enable, self.Menu_PrivateRoom_Disable, self.Menu_Empty2, self.Menu_PrivateRoom_Create, self.Menu_PrivateRoom_Disown, self.Menu_PrivateRoom_Dismember, self.Menu_Empty3, self.Menu_JoinPublicRoom, self.Menu_Empty4, self.Menu_Refresh = items
-
-        self.Menu_PrivateRoom_Enable.set_sensitive(False)
-        self.Menu_PrivateRoom_Disable.set_sensitive(False)
-        self.Menu_PrivateRoom_Create.set_sensitive(False)
+        self.Menu_Join, self.Menu_Leave, self.Menu_Create_Room, self.Menu_Empty1, self.Menu_PrivateRoom_Create, self.Menu_PrivateRoom_Disown, self.Menu_PrivateRoom_Dismember, self.Menu_Empty2, self.Menu_JoinPublicRoom, self.Menu_Empty3, self.Menu_Refresh = items
 
         self.frame.roomlist.RoomsList.connect("button_press_event", self.OnListClicked)
         self.frame.roomlist.RoomsList.set_headers_clickable(True)
@@ -148,8 +144,6 @@ class RoomsControl:
         self.ChatNotebook.Notebook.connect("switch-page", self.OnSwitchPage)
         self.ChatNotebook.Notebook.connect("page-reordered", self.OnReorderedPage)
 
-        self.frame.SetTextBG(self.frame.roomlist.RoomsList)
-        self.frame.SetTextBG(self.frame.roomlist.CreateRoomEntry)
         self.frame.SetTextBG(self.frame.roomlist.SearchRooms)
 
     def IsPrivateRoomOwned(self, room):
@@ -291,7 +285,6 @@ class RoomsControl:
         if event.button != 3 or self.roomsmodel is None:
             return
 
-        items = self.popup_menu.get_children()  # noqa: F841
         d = self.frame.roomlist.RoomsList.get_path_at_pos(int(event.x), int(event.y))
 
         if d:
@@ -320,8 +313,16 @@ class RoomsControl:
     def OnPopupJoin(self, widget):
         self.frame.np.queue.put(slskmessages.JoinRoom(self.popup_room))
 
-    def OnEnablePrivateRooms(self, widget):
-        self.frame.np.queue.put(slskmessages.PrivateRoomToggle(True))
+    def OnPopupCreatePublicRoom(self, widget):
+
+        room = EntryDialog(
+            self.frame.MainWindow,
+            title=_("Create Public Room"),
+            message=_('Enter the name of the public room you wish to create')
+        )
+
+        if room:
+            self.frame.np.queue.put(slskmessages.JoinRoom(room))
 
     def OnJoinPublicRoom(self, widget):
 
@@ -346,14 +347,11 @@ class RoomsControl:
 
         self.frame.np.queue.put(slskmessages.JoinPublicRoom())
 
-    def OnDisablePrivateRooms(self, widget):
-        self.frame.np.queue.put(slskmessages.PrivateRoomToggle(False))
-
     def OnPopupCreatePrivateRoom(self, widget):
 
-        room = input_box(
-            self.frame,
-            title=_('Nicotine+:') + " " + _("Create Private Room"),
+        room = EntryDialog(
+            self.frame.MainWindow,
+            title=_("Create Private Room"),
             message=_('Enter the name of the private room you wish to create')
         )
 
@@ -610,10 +608,6 @@ class RoomsControl:
 
         self.frame.np.config.sections["server"]["private_chatrooms"] = enabled
 
-        self.Menu_PrivateRoom_Enable.set_sensitive(not enabled)
-        self.Menu_PrivateRoom_Disable.set_sensitive(enabled)
-        self.Menu_PrivateRoom_Create.set_sensitive(enabled)
-
     def PrivateRoomDisown(self, msg):
         if msg.room in self.PrivateRooms:
             if self.PrivateRooms[msg.room]["owner"] == self.frame.np.config.sections["server"]["login"]:
@@ -680,8 +674,6 @@ class RoomsControl:
 
     def UpdateColours(self):
 
-        self.frame.SetTextBG(self.frame.roomlist.RoomsList)
-        self.frame.SetTextBG(self.frame.roomlist.CreateRoomEntry)
         self.frame.SetTextBG(self.frame.roomlist.SearchRooms)
 
         for room in list(self.joinedrooms.values()):
@@ -704,9 +696,6 @@ class RoomsControl:
 
         room = self.joinedrooms[msg.room]
 
-        if self.ChatNotebook.is_tab_detached(room.Main):
-            self.ChatNotebook.attach_tab(room.Main)
-
         self.ChatNotebook.remove_page(room.Main)
 
         room.destroy()
@@ -739,7 +728,7 @@ class RoomsControl:
                 clist += self.rooms
 
             if config["buddies"]:
-                clist += [i[0] for i in self.frame.userlist.userlist]
+                clist += [i[0] for i in self.frame.np.config.sections["server"]["userlist"]]
 
             if config["aliases"]:
                 clist += ["/" + k for k in list(self.frame.np.config.aliases.keys())]
@@ -753,146 +742,6 @@ class RoomsControl:
             room.GetCompletionList(clist=list(self.clist))
 
 
-class Ticker:
-
-    def __init__(self, TickerEventBox):
-
-        self.entry = gtk.Entry()
-        self.entry.set_property("editable", False)
-        self.entry.set_has_frame(False)
-        self.entry.show()
-
-        TickerEventBox.add(self.entry)
-
-        self.messages = []
-        self.ix = 0
-        self.source = None
-        self.shouldsort = False
-
-        self.enable()
-
-    def __del__(self):
-        GLib.source_remove(self.source)
-
-    def scroll(self, *args):
-
-        if not self.messages:
-            self.entry.set_text("")
-            return True
-
-        if self.shouldsort:
-            # A new ticker was added, sort the tickers by message length
-            self.messages.sort(key=lambda x: len(x[1]))
-            self.shouldsort = False
-
-        if self.ix >= len(self.messages):
-            self.ix = 0
-
-        (user, message) = self.messages[self.ix]
-        self.entry.set_text("[%s]: %s" % (user, message))
-        self.ix += 1
-
-        return True
-
-    def add_ticker(self, user, message):
-
-        message = message.replace("\n", " ")
-        self.messages.append((user, message))
-
-        # Indicates that the ticker list should be sorted on the next scroll
-        self.shouldsort = True
-
-    def remove_ticker(self, user, message):
-
-        try:
-            self.messages.remove((user, message))
-        except ValueError:
-            pass
-
-    def get_tickers(self):
-        return self.messages
-
-    def set_ticker(self, msgs):
-        self.messages = msgs
-        self.ix = 0
-        self.scroll()
-
-    def enable(self):
-
-        if self.source:
-            return
-
-        self.source = GLib.timeout_add(2500, self.scroll)
-
-    def disable(self):
-
-        if not self.source:
-            return
-
-        GLib.source_remove(self.source)
-
-        self.source = None
-
-
-def TickDialog(parent, default=""):
-
-    dlg = gtk.Dialog(
-        title=_("Nicotine+: Set ticker message"),
-        transient_for=parent
-    )
-    dlg.add_buttons(gtk.STOCK_CANCEL, gtk.ResponseType.CANCEL, gtk.STOCK_OK, gtk.ResponseType.OK)
-    dlg.set_default_response(gtk.ResponseType.OK)
-
-    t = 0
-
-    dlg.set_border_width(10)
-    dlg.vbox.set_spacing(15)
-
-    l = gtk.Label.new(_("Set room ticker message:"))  # noqa: E741
-    l.set_alignment(0, 0.5)
-    dlg.vbox.pack_start(l, False, False, 0)
-
-    entry = gtk.Entry()
-    entry.set_activates_default(True)
-    entry.set_text(default)
-    dlg.vbox.pack_start(entry, True, True, 0)
-
-    v = gtk.Box.new(gtk.Orientation.VERTICAL, False)
-    v.set_spacing(5)
-    r1 = gtk.RadioButton().new_from_widget(None)
-    r1.set_label(_("Just this time"))
-    r1.set_active(True)
-    v.pack_start(r1, False, False, 0)
-
-    r2 = gtk.RadioButton().new_from_widget(r1)
-    r2.set_label(_("Always for this channel"))
-    v.pack_start(r2, False, False, 0)
-
-    r3 = gtk.RadioButton().new_from_widget(r1)
-    r3.set_label(_("Default for all channels"))
-    v.pack_start(r3, False, False, 0)
-
-    dlg.vbox.pack_start(v, True, True, 0)
-
-    dlg.vbox.show_all()
-
-    result = None
-    if dlg.run() == gtk.ResponseType.OK:
-
-        if r1.get_active():
-            t = 0
-        elif r2.get_active():
-            t = 1
-        elif r3.get_active():
-            t = 2
-
-        result = entry.get_text()
-
-    dlg.destroy()
-
-    return [t, result]
-
-
 class ChatRoom:
 
     def __init__(self, roomsctrl, room, users, meta=False):
@@ -920,26 +769,18 @@ class ChatRoom:
 
         builder.connect_signals(self)
 
-        self.Ticker = Ticker(self.TickerEventBox)
+        self.Tickers = Tickers()
+        self.RoomWall = RoomWall(self.frame, self)
 
         self.room = room
         self.leaving = 0
         self.meta = meta  # not a real room if set to True
         config = self.frame.np.config.sections
 
-        if config["ticker"]["hide"]:
-            self.TickerEventBox.hide()
-        else:
-            self.TickerEventBox.show()
-
-        self.OnShowChatButtons(show=(not config["ui"]["chat_hidebuttons"]))
+        self.OnShowChatButtons()
 
         self.clist = []
 
-        self.Ticker.entry.connect("button_press_event", self.OnTickerClicked)
-        self.Ticker.entry.connect("focus-in-event", self.OnTickerFocus)
-        self.Ticker.entry.connect("focus-out-event", self.OnTickerFocus)
-
         if self.frame.gspell and self.frame.np.config.sections["ui"]["spellcheck"]:
             from gi.repository import Gspell
             spell_buffer = Gspell.EntryBuffer.get_from_gtk_entry_buffer(self.ChatEntry.get_buffer())
@@ -992,18 +833,7 @@ class ChatRoom:
         if len(config["columns"]["chatrooms"][room]) != 5:  # Insert new column to old settings.
             config["columns"]["chatrooms"][room].insert(1, 1)
 
-        try:
-            for i in range(len(cols)):
-
-                parent = cols[i].get_widget().get_ancestor(gtk.Button)
-                if parent:
-                    parent.connect("button_press_event", PressHeader)
-
-                # Read Show / Hide column settings from last session
-                cols[i].set_visible(config["columns"]["chatrooms"][room][i])
-        except IndexError:
-            # Column count in config is probably incorrect (outdated?), don't crash
-            pass
+        HideColumns(cols, config["columns"]["chatrooms"][room])
 
         if config["columns"]["hideflags"]:
             cols[1].set_visible(0)
@@ -1047,8 +877,6 @@ class ChatRoom:
         self.UpdateColours()
 
         self.UserList.set_model(self.usersmodel)
-        self.UserList.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, [('text/plain', 0, 2)], Gdk.DragAction.COPY)
-        self.UserList.connect("drag_data_get", self.drag_data_get_data)
 
         self.popup_menu_privaterooms = PopupMenu(self.frame, False)
         self.popup_menu = popup = PopupMenu(self.frame)
@@ -1091,7 +919,6 @@ class ChatRoom:
         self.UserList.connect("button_press_event", self.OnPopupMenu)
 
         self.ChatEntry.grab_focus()
-        self.ChatEntryBox.set_focus_child(self.ChatEntry)
 
         self.logpopupmenu = PopupMenu(self.frame).setup(
             ("#" + _("Find"), self.OnFindLogWindow),
@@ -1133,7 +960,7 @@ class ChatRoom:
         config = self.frame.np.config.sections
         log = os.path.join(
             config["logging"]["roomlogsdir"],
-            fixpath(self.room.replace(os.sep, "-")) + ".log"
+            CleanFile(self.room.replace(os.sep, "-")) + ".log"
         )
 
         try:
@@ -1142,20 +969,18 @@ class ChatRoom:
             numlines = 15
 
         try:
-            with open(log, 'r') as lines:
+            with open(log, 'r', encoding='utf-8') as lines:
                 # Only show as many log lines as specified in config
                 lines = deque(lines, numlines)
 
-                for bytes in lines:
-                    li = bytes
+                for line in lines:
 
                     # Try to parse line for username
-                    if len(li) > 20 and li[10].isspace() and li[11].isdigit() and li[20] in ("[", "*"):
+                    if len(line) > 20 and line[10].isspace() and line[11].isdigit() and line[20] in ("[", "*"):
 
-                        line = li[11:]
-                        if li[20] == "[" and li[20:].find("] ") != -1:
-                            namepos = li[20:].find("] ")
-                            user = li[21:20 + namepos].strip()
+                        if line[20] == "[" and line[20:].find("] ") != -1:
+                            namepos = line[20:].find("] ")
+                            user = line[21:20 + namepos].strip()
                             self.getUserTag(user)
                             usertag = self.tag_users[user]
                         else:
@@ -1164,14 +989,13 @@ class ChatRoom:
 
                         if user == config["server"]["login"]:
                             tag = self.tag_local
-                        elif li[20] == "*":
+                        elif line[20] == "*":
                             tag = self.tag_me
-                        elif li[20 + namepos:].upper().find(config["server"]["login"].upper()) > -1:
+                        elif line[20 + namepos:].upper().find(config["server"]["login"].upper()) > -1:
                             tag = self.tag_hilite
                         else:
                             tag = self.tag_remote
                     else:
-                        line = li
                         user = None
                         tag = None
                         usertag = None
@@ -1186,7 +1010,7 @@ class ChatRoom:
                 if len(lines) > 0:
                     AppendLine(self.ChatScroll, _("--- old messages above ---"), self.tag_hilite)
 
-        except IOError as e:  # noqa: F841
+        except IOError:
             pass
 
         GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
@@ -1210,12 +1034,6 @@ class ChatRoom:
     def OnFindChatLog(self, widget):
         self.frame.OnFindTextview(None, self.ChatScroll)
 
-    def drag_data_get_data(self, treeview, context, selection, target_id, etime):
-        treeselection = treeview.get_selection()
-        model, iter = treeselection.get_selected()
-        user = model.get_value(iter, 2)
-        selection.set(selection.target, 8, user)
-
     def destroy(self):
         self.Main.destroy()
 
@@ -1256,17 +1074,10 @@ class ChatRoom:
         if popup is None:
             return
 
-        menu = popup
-        user = menu.user  # noqa: F841
-        items = menu.get_children()  # noqa: F841
-
-        act = False  # noqa: F841
-
         return True
 
     def OnPopupMenu(self, widget, event):
 
-        items = self.popup_menu.get_children()  # noqa: F841
         d = self.UserList.get_path_at_pos(int(event.x), int(event.y))
 
         if not d:
@@ -1299,18 +1110,15 @@ class ChatRoom:
         self.popup_menu.editing = False
         self.popup_menu.popup(None, None, None, None, event.button, event.time)
 
+    def OnShowRoomWall(self, widget):
+        self.RoomWall.show()
+
     def OnShowChatHelp(self, widget):
-        self.frame.OnAboutChatroomCommands(widget, self.GetTabParent(self.Main))
+        self.frame.OnAboutChatroomCommands(widget)
 
     def OnShowChatButtons(self, show=True):
 
-        for widget in self.HideStatusLog, self.HideUserList, self.ShowChatHelp:
-            if show:
-                widget.show()
-            else:
-                widget.hide()
-
-        if self.frame.np.config.sections["ui"]["speechenabled"] and show:
+        if self.frame.np.config.sections["ui"]["speechenabled"]:
             self.Speech.show()
         else:
             self.Speech.hide()
@@ -1320,30 +1128,30 @@ class ChatRoom:
         act = widget.get_active()
         if act:
             self.RoomLogWindow.hide()
-            self.HideStatusLogImage.set_from_stock(gtk.STOCK_GO_DOWN, 1)
+            self.HideStatusLogImage.set_from_icon_name("go-down-symbolic", gtk.IconSize.BUTTON)
         else:
             self.RoomLogWindow.show()
-            self.HideStatusLogImage.set_from_stock(gtk.STOCK_GO_UP, 1)
+            self.HideStatusLogImage.set_from_icon_name("go-up-symbolic", gtk.IconSize.BUTTON)
 
     def OnHideUserList(self, widget):
 
         act = widget.get_active()
         if act:
             self.vbox5.hide()
-            self.HideUserListImage.set_from_stock(gtk.STOCK_GO_BACK, 1)
+            self.HideUserListImage.set_from_icon_name("go-previous-symbolic", gtk.IconSize.BUTTON)
         else:
             self.vbox5.show()
-            self.HideUserListImage.set_from_stock(gtk.STOCK_GO_FORWARD, 1)
+            self.HideUserListImage.set_from_icon_name("go-next-symbolic", gtk.IconSize.BUTTON)
 
     def TickerSet(self, msg):
 
-        self.Ticker.set_ticker([])
+        self.Tickers.set_ticker([])
         for user in list(msg.msgs.keys()):
             if user in self.frame.np.config.sections["server"]["ignorelist"] or self.frame.UserIpIsIgnored(user):
                 # User ignored, ignore Ticker messages
                 return
 
-            self.Ticker.add_ticker(user, msg.msgs[user])
+            self.Tickers.add_ticker(user, msg.msgs[user])
 
     def TickerAdd(self, msg):
 
@@ -1352,10 +1160,10 @@ class ChatRoom:
             # User ignored, ignore Ticker messages
             return
 
-        self.Ticker.add_ticker(msg.user, msg.msg)
+        self.Tickers.add_ticker(msg.user, msg.msg)
 
     def TickerRemove(self, msg):
-        self.Ticker.remove_ticker(msg.user, msg.msg)
+        self.Tickers.remove_ticker(msg.user)
 
     def SayChatRoom(self, msg, text, public=False):
         text = re.sub("\\s\\s+", "  ", text)
@@ -1374,26 +1182,34 @@ class ChatRoom:
             if tag == self.tag_hilite:
 
                 self.roomsctrl.ChatNotebook.request_hilite(self.Main)
+                self.frame.ChatRequestIcon(1, self.Main)
 
-                if self.roomsctrl.ChatNotebook.is_tab_detached(self.Main):
-                    if not self.roomsctrl.ChatNotebook.is_detached_tab_focused(self.Main):
-                        self.frame.Notifications.Add("rooms", user, self.room, tab=False)
-                else:
-                    self.frame.ChatRequestIcon(1, self.Main)
-
-                    # add hilite to trayicon
-                    if self.roomsctrl.ChatNotebook.get_current_page() != self.roomsctrl.ChatNotebook.page_num(self.roomsctrl.joinedrooms[self.room].Main) or \
-                       self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.chathbox) or \
-                       not self.frame.MainWindow.get_property("visible"):
-                        if self.room not in self.frame.hilites["rooms"]:
-                            self.frame.Notifications.Add("rooms", user, self.room, tab=True)
+                if self.frame.np.config.sections["notifications"]["notification_popup_chatroom_mention"]:
+                    self.frame.Notifications.NewNotificationPopup(
+                        text,
+                        title=_("%s mentioned you in the %s room") % (user, self.room),
+                        soundnamenotify="bell-window-system",
+                        soundnamewin="SystemExclamation"
+                    )
 
             else:
                 self.roomsctrl.ChatNotebook.request_changed(self.Main)
-                if self.roomsctrl.ChatNotebook.is_tab_detached(self.Main):
-                    pass
-                else:
-                    self.frame.ChatRequestIcon(0)
+                self.frame.ChatRequestIcon(0)
+
+            if self.roomsctrl.ChatNotebook.get_current_page() != self.roomsctrl.ChatNotebook.page_num(self.roomsctrl.joinedrooms[self.room].Main) or \
+                self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.chathbox) or \
+                    not self.frame.MainWindow.get_property("visible"):
+
+                if tag == self.tag_hilite:
+
+                    if self.room not in self.frame.hilites["rooms"]:
+                        self.frame.Notifications.Add("rooms", user, self.room, tab=True)
+
+                elif self.frame.np.config.sections["notifications"]["notification_popup_chatroom"]:
+                    self.frame.Notifications.NewNotificationPopup(
+                        text,
+                        title=_("Message by %s in the %s room") % (user, self.room)
+                    )
 
         if text[:4] == "/me ":
 
@@ -1607,13 +1423,6 @@ class ChatRoom:
         elif cmd == "/now":
             self.NowPlayingThread()
 
-        elif cmd == "/detach":
-            self.Detach()
-
-        elif cmd == "/attach":
-            self.roomsctrl.ChatNotebook.attach_tab(self.Main)
-            GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
-
         elif cmd == "/rescan":
 
             # Rescan public shares if needed
@@ -1655,13 +1464,9 @@ class ChatRoom:
         self.ChatEntry.set_text("")
 
     def showTickers(self):
-        tickers = self.Ticker.get_tickers()
-        header = _("All ticker messages for %(room)s:") % {'room': self.room}
-        self.frame.logMessage("%s\n%s" % (header, "\n".join(["%s: %s" % (user, msg) for (user, msg) in tickers])))
-
-    def Detach(self, widget=None):
-        self.roomsctrl.ChatNotebook.detach_tab(self.Main, _("Nicotine+ Chatroom: %s") % self.room)
-        GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
+        tickers = self.Tickers.get_tickers()
+        header = _("All tickers / wall messages for %(room)s:") % {'room': self.room}
+        self.frame.logMessage("%s\n%s" % (header, "\n".join(["[%s] %s" % (user, msg) for (user, msg) in tickers])))
 
     def Say(self, text):
         text = re.sub("\\s\\s+", "  ", text)
@@ -1714,7 +1519,7 @@ class ChatRoom:
         # Remove from completion list, and completion drop-down
         if self.frame.np.config.sections["words"]["tab"]:
 
-            if username in self.clist and username not in [i[0] for i in self.frame.userlist.userlist]:
+            if username in self.clist and username not in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]]:
 
                 self.clist.remove(username)
 
@@ -1741,12 +1546,9 @@ class ChatRoom:
     def CountUsers(self):
 
         numusers = len(self.users)
-        if numusers > 1:
+        if numusers > 0:
             self.LabelPeople.show()
-            self.LabelPeople.set_text(_("%i people in room") % numusers)
-        elif numusers == 1:
-            self.LabelPeople.show()
-            self.LabelPeople.set_text(_("You are alone"))
+            self.LabelPeople.set_text(_("User List (%i)") % numusers)
         else:
             self.LabelPeople.hide()
 
@@ -1848,13 +1650,18 @@ class ChatRoom:
                 tag.set_property("underline", pango.Underline.NONE)
 
             tag.connect("event", self.UserNameEvent, username)
-            tag.last_event_type = -1
 
         return tag
 
     def UserNameEvent(self, tag, widget, event, iter, user):
 
-        if tag.last_event_type == Gdk.EventType.BUTTON_PRESS and event.button.type == Gdk.EventType.BUTTON_RELEASE and event.button.button in (1, 2):
+        """
+        Mouse buttons:
+        1 = left button
+        2 = middle button
+        3 = right button
+        """
+        if event.button.type == Gdk.EventType.BUTTON_PRESS and event.button.button in (1, 2, 3):
 
             # Chat, Userlists use the normal popup system
             self.popup_menu.editing = True
@@ -1873,7 +1680,7 @@ class ChatRoom:
             self.popup_menu.editing = False
             self.popup_menu.popup(None, None, None, None, event.button.button, event.button.time)
 
-        tag.last_event_type = event.button.type
+        return True
 
     def UpdateColours(self):
 
@@ -1893,14 +1700,8 @@ class ChatRoom:
         logbuffer = self.RoomLog.get_buffer()
         self.tag_log = self.makecolour(logbuffer, "chatremote")
 
-        self.frame.SetTextBG(self.ChatScroll)
-        self.frame.SetTextBG(self.RoomLog)
-        self.frame.SetTextBG(self.UserList)
-
         self.frame.SetTextBG(self.ChatEntry)
-        self.frame.SetTextBG(self.AutoJoin)
-        self.frame.SetTextBG(self.Log)
-        self.frame.SetTextBG(self.Ticker.entry)
+        self.frame.SetTextBG(self.RoomWall.RoomWallEntry)
 
     def getUserStatusColor(self, status):
 
@@ -1959,13 +1760,8 @@ class ChatRoom:
         for user in self.tag_users:
             self.getUserTag(user)
 
-        self.frame.SetTextBG(self.ChatScroll)
-        self.frame.SetTextBG(self.RoomLog)
-        self.frame.SetTextBG(self.UserList)
         self.frame.SetTextBG(self.ChatEntry)
-        self.frame.SetTextBG(self.AutoJoin)
-        self.frame.SetTextBG(self.Log)
-        self.frame.SetTextBG(self.Ticker.entry)
+        self.frame.SetTextBG(self.RoomWall.RoomWallEntry)
         self.frame.ChangeListFont(self.UserList, self.frame.np.config.sections["ui"]["listfont"])
 
     def OnLeave(self, widget=None):
@@ -1973,7 +1769,6 @@ class ChatRoom:
         if self.leaving:
             return
 
-        self.Leave.set_sensitive(False)
         self.leaving = 1
 
         config = self.frame.np.config.sections
@@ -2024,7 +1819,7 @@ class ChatRoom:
         for tag in list(self.tag_users.values()):
             self.changecolour(tag, "useroffline")
 
-        self.Ticker.set_ticker([])
+        self.Tickers.set_ticker([])
 
     def Rejoined(self, users):
 
@@ -2271,68 +2066,6 @@ class ChatRoom:
     def OnClearRoomLog(self, widget):
         self.RoomLog.get_buffer().set_text("")
 
-    def OnTickerFocus(self, widget, event):
-
-        if widget.is_focus():
-            self.Ticker.disable()
-        else:
-            self.Ticker.enable()
-
-    def GetTabParent(self, page):
-
-        if self.roomsctrl.ChatNotebook.is_tab_detached(page):
-            return self.Main.get_parent().get_parent()
-
-        return self.frame.MainWindow
-
-    def OnTickerClicked(self, widget, event):
-
-        if event.button != 1 or event.type != Gdk.EventType._2BUTTON_PRESS:
-            return False
-
-        config = self.frame.np.config.sections
-
-        if config["server"]["login"] in self.Ticker.messages:
-            old = self.Ticker.messages[config["server"]["login"]]
-        else:
-            old = ""
-
-        t, result = TickDialog(self.GetTabParent(self.Main), old)
-
-        if result is not None:
-
-            if t == 1:
-
-                if not result:
-                    if self.room in config["ticker"]["rooms"]:
-                        del config["ticker"]["rooms"][self.room]
-                else:
-                    config["ticker"]["rooms"][self.room] = result
-
-                self.frame.np.config.writeConfiguration()
-
-            elif t == 2:
-
-                if self.room in config["ticker"]["rooms"]:
-                    del config["ticker"]["rooms"][self.room]
-
-                config["ticker"]["default"] = result
-
-                self.frame.np.config.writeConfiguration()
-
-            self.frame.np.queue.put(slskmessages.RoomTickerSet(self.room, result))
-
-        return True
-
-    def ShowTicker(self, visible):
-
-        if visible:
-            self.Ticker.enable()
-            self.TickerEventBox.show()
-        else:
-            self.Ticker.disable()
-            self.TickerEventBox.hide()
-
 
 class ChatRooms(IconNotebook):
 
@@ -2340,15 +2073,15 @@ class ChatRooms(IconNotebook):
 
         self.frame = frame
 
-        ui = self.frame.np.config.sections["ui"]
+        config = self.frame.np.config.sections
 
         IconNotebook.__init__(
             self,
             self.frame.images,
-            angle=ui["labelrooms"],
-            tabclosers=ui["tabclosers"],
-            show_image=ui["tab_icons"],
-            reorderable=ui["tab_reorderable"],
+            angle=config["ui"]["labelrooms"],
+            tabclosers=config["ui"]["tabclosers"],
+            show_image=config["notifications"]["notification_tab_icons"],
+            reorderable=config["ui"]["tab_reorderable"],
             notebookraw=self.frame.ChatNotebookRaw
         )
 
@@ -2356,7 +2089,7 @@ class ChatRooms(IconNotebook):
 
         self.popup_enable()
 
-        self.set_tab_pos(self.frame.getTabPosition(self.frame.np.config.sections["ui"]["tabrooms"]))
+        self.set_tab_pos(self.frame.getTabPosition(config["ui"]["tabrooms"]))
 
     def TabPopup(self, room):
 
@@ -2365,7 +2098,6 @@ class ChatRooms(IconNotebook):
 
         popup = PopupMenu(self.frame)
         popup.setup(
-            ("#" + _("Detach this tab"), self.roomsctrl.joinedrooms[room].Detach),
             ("#" + _("Leave this room"), self.roomsctrl.joinedrooms[room].OnLeave)
         )
         popup.set_user(room)
