@@ -30,7 +30,6 @@ lookup = {
     'ai': 'Anguilla',
     'al': 'Albania',
     'am': 'Armenia',
-    'an': 'Netherlands Antilles',
     'ao': 'Angola',
     'aq': 'Antarctica',
     'ar': 'Argentina',
@@ -38,6 +37,7 @@ lookup = {
     'at': 'Austria',
     'au': 'Australia',
     'aw': 'Aruba',
+    'ax': 'Åland Islands',
     'az': 'Azerbaijan',
     'ba': 'Bosnia & Herzegovina',
     'bb': 'Barbados',
@@ -48,9 +48,11 @@ lookup = {
     'bh': 'Bahrain',
     'bi': 'Burundi',
     'bj': 'Benin',
+    'bl': 'Saint Barthelemy',
     'bm': 'Bermuda',
     'bn': 'Brunei Darussalam',
     'bo': 'Bolivia',
+    'bq': 'Bonaire, Sint Eustatius and Saba',
     'br': 'Brazil',
     'bs': 'Bahamas',
     'bt': 'Bhutan',
@@ -71,9 +73,9 @@ lookup = {
     'cn': 'China',
     'co': 'Colombia',
     'cr': 'Costa Rica',
-    'cs': 'Czechoslovakia (former)',
     'cu': 'Cuba',
-    'cv': 'Cape Verde',
+    'cv': 'Cabo Verde',
+    'cw': 'Curaçao',
     'cx': 'Christmas Island',
     'cy': 'Cyprus',
     'cz': 'Czech Republic',
@@ -102,6 +104,7 @@ lookup = {
     'gd': 'Grenada',
     'ge': 'Georgia',
     'gf': 'French Guiana',
+    'gg': 'Guernsey',
     'gh': 'Ghana',
     'gi': 'Gibraltar',
     'gl': 'Greenland',
@@ -124,12 +127,14 @@ lookup = {
     'id': 'Indonesia',
     'ie': 'Ireland',
     'il': 'Israel',
+    'im': 'Isle of Man',
     'in': 'India',
     'io': 'British Indian Ocean Territory',
     'iq': 'Iraq',
     'ir': 'Iran',
     'is': 'Iceland',
     'it': 'Italy',
+    'je': 'Jersey',
     'jm': 'Jamaica',
     'jo': 'Jordan',
     'jp': 'Japan',
@@ -158,9 +163,11 @@ lookup = {
     'ma': 'Morocco',
     'mc': 'Monaco',
     'md': 'Moldova',
+    'me': 'Montenegro',
+    'mf': 'Saint Martin',
     'mg': 'Madagascar',
     'mh': 'Marshall Islands',
-    'mk': 'Macedonia',
+    'mk': 'North Macedonia',
     'ml': 'Mali',
     'mm': 'Myanmar',
     'mn': 'Mongolia',
@@ -199,13 +206,14 @@ lookup = {
     'pm': 'Saint Pierre & Miquelon',
     'pn': 'Pitcairn',
     'pr': 'Puerto Rico',
-    'ps': 'Occupied Palestinian Territory',
+    'ps': 'State of Palestine',
     'pt': 'Portugal',
     'pw': 'Palau',
     'py': 'Paraguay',
     'qa': 'Qatar',
     're': 'Reunion',
     'ro': 'Romania',
+    'rs': 'Serbia',
     'ru': 'Russia',
     'rw': 'Rwanda',
     'sa': 'Saudi Arabia',
@@ -223,10 +231,12 @@ lookup = {
     'sn': 'Senegal',
     'so': 'Somalia',
     'sr': 'Suriname',
+    'ss': 'South Sudan',
     'st': 'Sao Tome & Principe',
     'sv': 'El Salvador',
+    'sx': 'Sint Maarten',
     'sy': 'Syria',
-    'sz': 'Swaziland',
+    'sz': 'Eswatini',
     'tc': 'Turks & Caicos Islands',
     'td': 'Chad',
     'tf': 'French Southern Territories',
@@ -238,7 +248,6 @@ lookup = {
     'tm': 'Turkmenistan',
     'tn': 'Tunisia',
     'to': 'Tonga',
-    'tp': 'East Timor',
     'tr': 'Turkey',
     'tt': 'Trinidad & Tobago',
     'tv': 'Tuvalu',
@@ -259,9 +268,9 @@ lookup = {
     'vu': 'Vanuatu',
     'wf': 'Wallis & Futuna',
     'ws': 'Samoa',
+    'xk': 'Kosovo',
     'ye': 'Yemen',
     'yt': 'Mayotte',
-    'yu': 'Yugoslavia (former)',
     'za': 'South Africa',
     'zm': 'Zambia',
     'zw': 'Zimbabwe',
