@@ -26,7 +26,6 @@ import re
 import sys
 from gettext import gettext as _
 
-from gi.repository import Gdk
 from gi.repository import GObject as gobject
 from gi.repository import Gtk as gtk
 
@@ -41,7 +40,6 @@ class NowPlaying:
 
         # Build the window
         self.frame = frame
-        self.accel_group = gtk.AccelGroup()
 
         builder = gtk.Builder()
 
@@ -58,14 +56,11 @@ class NowPlaying:
             except TypeError:
                 pass
 
-        self.NowPlaying.set_icon(self.frame.images["n"])
         self.NowPlaying.set_transient_for(self.frame.MainWindow)
-        self.NowPlaying.add_accel_group(self.accel_group)
 
         self.NowPlaying.connect("destroy", self.quit)
         self.NowPlaying.connect("destroy-event", self.quit)
         self.NowPlaying.connect("delete-event", self.quit)
-        self.NowPlaying.connect("key-press-event", self.OnKeyPress)
 
         self.title_clear()
 
@@ -229,12 +224,6 @@ class NowPlaying:
     def OnNPCancel(self, widget):
         self.quit(None)
 
-    def OnKeyPress(self, widget, event):
-
-        # Close the window when escape is pressed
-        if event.keyval == Gdk.KEY_Escape:
-            self.quit(None)
-
     def quit(self, widget, s=None):
 
         # Save new defined formats in npformatlist before exiting
@@ -651,20 +640,20 @@ class NowPlaying:
         import json
 
         try:
-            conn = http.client.HTTPConnection("ws.audioscrobbler.com")
+            conn = http.client.HTTPSConnection("ws.audioscrobbler.com")
         except Exception as error:
             log.addwarning(_("ERROR: lastfm: Could not connect to audioscrobbler: %(error)s") % {"error": error})
             return None
 
         try:
             (user, apikey) = self.NPCommand.get_text().split(';')
-        except ValueError as error:  # noqa: F841
+        except ValueError:
             log.addwarning(_("ERROR: lastfm: Please provide both your lastfm username and API key"))
             return None
 
-        conn.request("GET", "/2.0/?method=user.getrecenttracks&user=" + user + "&api_key=" + apikey + "&format=json")
+        conn.request("GET", "/2.0/?method=user.getrecenttracks&user=" + user + "&api_key=" + apikey + "&format=json", headers={"User-Agent": "Nicotine+"})
         resp = conn.getresponse()
-        data = resp.read()
+        data = resp.read().decode("utf-8")
 
         if resp.status != 200 or resp.reason != "OK":
             log.addwarning(_("ERROR: lastfm: Could not get recent track from audioscrobbler: %(error)s") % {"error": str(data)})
