@@ -41,9 +41,9 @@ from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import ScrollBottom
 from pynicotine.gtkgui.utils import WriteLog
 from pynicotine.gtkgui.utils import expand_alias
-from pynicotine.gtkgui.utils import fixpath
 from pynicotine.gtkgui.utils import is_alias
 from pynicotine.logfacility import log
+from pynicotine.utils import CleanFile
 from pynicotine.utils import version
 
 
@@ -56,7 +56,7 @@ class PrivateChats(IconNotebook):
         [
             "/alias ", "/unalias ", "/whois ", "/browse ", "/ip ", "/pm ", "/msg ", "/search ", "/usearch ", "/rsearch ",
             "/bsearch ", "/join ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/ignoreip ", "/unban ", "/unignore ",
-            "/clear ", "/quit ", "/exit ", "/rescan ", "/info ", "/attach ", "/detach ", "/ctcpversion "
+            "/clear ", "/quit ", "/exit ", "/rescan ", "/info ", "/ctcpversion "
         ]
     )
 
@@ -64,16 +64,16 @@ class PrivateChats(IconNotebook):
 
         self.frame = frame
 
-        ui = frame.np.config.sections["ui"]
+        config = frame.np.config.sections
 
         IconNotebook.__init__(
             self,
             self.frame.images,
-            angle=ui["labelprivate"],
-            tabclosers=ui["tabclosers"],
-            show_image=ui["tab_icons"],
-            reorderable=ui["tab_reorderable"],
-            show_status_image=ui["tab_status_icons"],
+            angle=config["ui"]["labelprivate"],
+            tabclosers=config["ui"]["tabclosers"],
+            show_image=config["notifications"]["notification_tab_icons"],
+            reorderable=config["ui"]["tab_reorderable"],
+            show_status_image=config["ui"]["tab_status_icons"],
             notebookraw=self.frame.PrivatechatNotebookRaw
         )
 
@@ -171,7 +171,6 @@ class PrivateChats(IconNotebook):
             ("$" + _("B_lock this user's IP Address"), popup.OnBlockUser),
             ("$" + _("Ignore this user's IP Address"), popup.OnIgnoreIP),
             ("", None),
-            ("#" + _("Detach this tab"), self.users[user].Detach),
             ("#" + _("Close this tab"), self.users[user].OnClose)
         )
 
@@ -235,21 +234,21 @@ class PrivateChats(IconNotebook):
         chat = self.users[msg.user]
         self.request_changed(chat.Main)
 
-        if self.is_tab_detached(chat.Main):
-            # Only show notifications if window is not focused, and don't highlight main window
-            if not self.is_detached_tab_focused(chat.Main):
-                self.frame.Notifications.Add("private", msg.user)
-        else:
-            # If tab isn't detached
-            # Hilight main private chats Label
-            self.frame.RequestIcon(self.frame.PrivateChatTabLabel, chat.Main)
+        # Hilight main private chats Label
+        self.frame.RequestIcon(self.frame.PrivateChatTabLabel, chat.Main)
 
-            # Show notifications if the private chats notebook isn't selected,
-            # the tab is not selected, or the main window isn't mapped
-            if self.get_current_page() != self.page_num(chat.Main) or \
-               self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox) or \
-               not self.frame.MainWindow.get_property("visible"):
-                self.frame.Notifications.Add("private", msg.user)
+        # Show notifications if the private chats notebook isn't selected,
+        # the tab is not selected, or the main window isn't mapped
+        if self.get_current_page() != self.page_num(chat.Main) or \
+           self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox) or \
+           not self.frame.MainWindow.get_property("visible"):
+            self.frame.Notifications.Add("private", msg.user)
+
+            if self.frame.np.config.sections["notifications"]["notification_popup_private_message"]:
+                self.frame.Notifications.NewNotificationPopup(
+                    text,
+                    title=_("Private message from %s") % msg.user
+                )
 
         # SEND CLIENT VERSION to user if the following string is sent
         ctcpversion = 0
@@ -281,11 +280,8 @@ class PrivateChats(IconNotebook):
         if tab.user in self.frame.np.config.sections["privatechat"]["users"]:
             self.frame.np.config.sections["privatechat"]["users"].remove(tab.user)
 
-        if self.is_tab_detached(tab.Main):
-            tab.Main.get_parent_window().destroy()
-        else:
-            self.remove_page(tab.Main)
-            tab.Main.destroy()
+        self.remove_page(tab.Main)
+        tab.Main.destroy()
 
     def Login(self):
 
@@ -315,7 +311,7 @@ class PrivateChats(IconNotebook):
         clist = [self.frame.np.config.sections["server"]["login"], "nicotine"] + list(self.users.keys())
 
         if config["buddies"]:
-            clist += [i[0] for i in self.frame.userlist.userlist]
+            clist += [i[0] for i in self.frame.np.config.sections["server"]["userlist"]]
 
         if config["aliases"]:
             clist += ["/" + k for k in list(self.frame.np.config.aliases.keys())]
@@ -427,7 +423,7 @@ class PrivateChat:
 
         # Read log file
         config = self.frame.np.config.sections
-        log = os.path.join(config["logging"]["privatelogsdir"], fixpath(self.user.replace(os.sep, "-")) + ".log")
+        log = os.path.join(config["logging"]["privatelogsdir"], CleanFile(self.user.replace(os.sep, "-")) + ".log")
 
         try:
             numlines = int(config["logging"]["readprivatelines"])
@@ -435,13 +431,13 @@ class PrivateChat:
             numlines = 15
 
         try:
-            with open(log, 'r') as lines:
+            with open(log, 'r', encoding='utf-8') as lines:
                 # Only show as many log lines as specified in config
                 lines = deque(lines, numlines)
 
                 for line in lines:
                     AppendLine(self.ChatScroll, line, self.tag_hilite, timestamp_format="", username=self.user, usertag=self.tag_hilite)
-        except IOError as e:  # noqa: F841
+        except IOError:
             pass
 
         GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
@@ -510,6 +506,9 @@ class PrivateChat:
     def OnClearChatLog(self, widget):
         self.ChatScroll.get_buffer().set_text("")
 
+    def OnShowChatHelp(self, widget):
+        self.frame.OnAboutPrivateChatCommands(widget)
+
     def ShowMessage(self, text, status=None, timestamp=None):
 
         self.UpdateColours()
@@ -755,12 +754,6 @@ class PrivateChat:
         elif cmd == "/now":
             self.NowPlayingThread()
 
-        elif cmd == "/detach":
-            self.Detach()
-
-        elif cmd == "/attach":
-            self.Attach()
-
         elif cmd == "/rescan":
             self.frame.OnRescan()
 
@@ -784,25 +777,11 @@ class PrivateChat:
 
         widget.set_text("")
 
-    def Attach(self, widget=None):
-        self.chats.attach_tab(self.Main)
-        GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
-
-    def Detach(self, widget=None):
-        self.chats.detach_tab(
-            self.Main,
-            _("Nicotine+ Private Chat: %(user)s (%(status)s)") % {
-                'user': self.user,
-                'status': [_("Offline"), _("Away"), _("Online")][self.status]
-            }
-        )
-        GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
-
     def NowPlayingThread(self):
         if self.frame.now is None:
             self.frame.now = nowplaying.NowPlaying(self.frame)
 
-        np = self.frame.now.DisplayNowPlaying(None, 0, self.SendMessage)  # noqa: F841
+        self.frame.now.DisplayNowPlaying(None, 0, self.SendMessage)
 
     def makecolour(self, buffer, colour):
 
@@ -864,9 +843,7 @@ class PrivateChat:
             self.tag_username.set_property("underline", pango.Underline.NONE)
             self.tag_my_username.set_property("underline", pango.Underline.NONE)
 
-        self.frame.SetTextBG(self.ChatScroll)
         self.frame.SetTextBG(self.ChatLine)
-        self.frame.SetTextBG(self.Log)
         self.frame.SetTextBG(self.PeerPrivateMessages)
 
     def changecolour(self, tag, colour):
@@ -918,7 +895,6 @@ class PrivateChat:
         else:
             self.changecolour(self.tag_my_username, "useroffline")
 
-        self.frame.SetTextBG(self.ChatScroll)
         self.frame.SetTextBG(self.ChatLine)
 
     def getUserStatusColor(self, status):
@@ -943,9 +919,6 @@ class PrivateChat:
         self.status = status
         color = self.getUserStatusColor(self.status)
         self.changecolour(self.tag_username, color)
-        statusword = [_("Offline"), _("Away"), _("Online")][status]
-        title = _("Nicotine+ Private Chat: %(user)s (%(status)s)") % {'user': self.user, 'status': statusword}
-        self.chats.set_detached_tab_title(self.Main, title)
 
     def OnClose(self, widget):
 
