@@ -21,44 +21,67 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 from gettext import gettext as _
-from math import ceil
+from sys import maxsize
 from time import time
 
-from gi.repository import GLib
 from gi.repository import GObject as gobject
 from gi.repository import Gtk as gtk
 
+from pynicotine.gtkgui.utils import HideColumns
 from pynicotine.gtkgui.utils import HumanSize
 from pynicotine.gtkgui.utils import HumanSpeed
 from pynicotine.gtkgui.utils import InitialiseColumns
 from pynicotine.gtkgui.utils import PopupMenu
-from pynicotine.gtkgui.utils import PressHeader
 from pynicotine.gtkgui.utils import SelectUserRowIter
 
 
 class TransferList:
 
-    MINIMUM_GUI_DELAY = 1  # in seconds
-    MINIMUM_GUI_DELAY_SLEEP = int(ceil(MINIMUM_GUI_DELAY * 2000))  # in ms
-
     def __init__(self, frame, widget, type):
         self.frame = frame
         self.widget = widget
         self.type = type
+        self.last_ui_update = self.last_save = time()
         self.list = None
-        self.selected_transfers = []
-        self.selected_users = []
         self.users = {}
         self.paths = {}
-        self.lastupdate = 0
-        self.finalupdatetimerid = None
+
+        # Status list
+        self.statuses = {}
+        self.statuses["Waiting for download"] = _("Waiting for download")
+        self.statuses["Requesting file"] = _("Requesting file")
+        self.statuses["Initializing transfer"] = _("Initializing transfer")
+        self.statuses["Cannot connect"] = _("Cannot connect")
+        self.statuses["Waiting for peer to connect"] = _("Waiting for peer to connect")
+        self.statuses["Connecting"] = _("Connecting")
+        self.statuses["Getting address"] = _("Getting address")
+        self.statuses["Getting status"] = _("Getting status")
+        self.statuses["Queued"] = _("Queued")
+        self.statuses["User logged off"] = _("User logged off")
+        self.statuses["Aborted"] = _("Aborted")
+        self.statuses["Finished"] = _("Finished")
+        self.statuses["Paused"] = _("Paused")
+        self.statuses["Transferring"] = _("Transferring")
+        self.statuses["Filtered"] = _("Filtered")
+        self.statuses["Connection closed by peer"] = _("Connection closed by peer")
+        self.statuses["File not shared"] = _("File not shared")
+        self.statuses["File not shared."] = _("File not shared")  # The official client sends a variant containing a dot
+        self.statuses["Establishing connection"] = _("Establishing connection")
+        self.statuses["Download directory error"] = _("Download directory error")
+        self.statuses["Local file error"] = _("Local file error")
+        self.statuses["Remote file error"] = _("Remote file error")
+
+        # String templates
+        self.extension_list_template = _("All %(ext)s")
+        self.files_template = _("%(number)2s files ")
 
         widget.get_selection().set_mode(gtk.SelectionMode.MULTIPLE)
         widget.set_enable_tree_lines(True)
         widget.set_rubber_banding(True)
 
-        columntypes = [
+        self.transfersmodel = gtk.TreeStore(
             str,                   # (0)  user
             str,                   # (1)  path
             str,                   # (2)  file name
@@ -77,39 +100,28 @@ class TransferList:
             gobject.TYPE_UINT64,   # (15) time elapsed
             gobject.TYPE_UINT64,   # (16) file count
             gobject.TYPE_UINT64,   # (17) queue position
-        ]
+        )
+
+        text_color = self.frame.np.config.sections["ui"]["search"]
 
-        self.transfersmodel = gtk.TreeStore(*columntypes)
         widths = self.frame.np.config.sections["columns"]["{}_widths".format(type)]
-
         self.cols = cols = InitialiseColumns(
             widget,
-            [_("User"), widths[0], "text", self.CellDataFunc],
-            [_("Path"), widths[1], "text", self.CellDataFunc],
-            [_("Filename"), widths[2], "text", self.CellDataFunc],
-            [_("Status"), widths[3], "text", self.CellDataFunc],
-            [_("Queue Position"), widths[4], "text", self.CellDataFunc],
+            [_("User"), widths[0], "text", None, (text_color, None)],
+            [_("Path"), widths[1], "text", None, (text_color, None)],
+            [_("Filename"), widths[2], "text", None, (text_color, None)],
+            [_("Status"), widths[3], "text", None, (text_color, None)],
+            [_("Queue Position"), widths[4], "number", None, (text_color, None)],
             [_("Percent"), widths[5], "progress"],
-            [_("Size"), widths[6], "text", self.CellDataFunc],
-            [_("Speed"), widths[7], "text", self.CellDataFunc],
-            [_("Time elapsed"), widths[8], "text", self.CellDataFunc],
-            [_("Time left"), widths[9], "text", self.CellDataFunc]
+            [_("Size"), widths[6], "number", None, (text_color, None)],
+            [_("Speed"), widths[7], "number", None, (text_color, None)],
+            [_("Time elapsed"), widths[8], "number", None, (text_color, None)],
+            [_("Time left"), widths[9], "number", None, (text_color, None)],
         )
 
         self.col_user, self.col_path, self.col_filename, self.col_status, self.col_position, self.col_percent, self.col_human_size, self.col_human_speed, self.col_time_elapsed, self.col_time_left = cols
 
-        try:
-            for i in range(len(cols)):
-
-                parent = cols[i].get_widget().get_ancestor(gtk.Button)
-                if parent:
-                    parent.connect("button_press_event", PressHeader)
-
-                # Read Show / Hide column settings from last session
-                cols[i].set_visible(self.frame.np.config.sections["columns"][self.type + "_columns"][i])
-        except IndexError:
-            # Column count in config is probably incorrect (outdated?), don't crash
-            pass
+        HideColumns(cols, self.frame.np.config.sections["columns"][self.type + "_columns"])
 
         self.col_user.set_sort_column_id(0)
         self.col_path.set_sort_column_id(1)
@@ -141,17 +153,8 @@ class TransferList:
         self.frame.np.config.sections["columns"][self.type + "_widths"] = widths
 
     def UpdateColours(self):
-        self.frame.SetTextBG(self.widget)
         self.frame.ChangeListFont(self.widget, self.frame.np.config.sections["ui"]["transfersfont"])
 
-    def CellDataFunc(self, column, cellrenderer, model, iter, dummy="dummy"):
-
-        colour = self.frame.np.config.sections["ui"]["search"]
-        if colour == "":
-            colour = None
-
-        cellrenderer.set_property("foreground", colour)
-
     def InitInterface(self, list):
         self.list = list
         self.update()
@@ -163,8 +166,8 @@ class TransferList:
         self.Clear()
 
     def select_transfers(self):
-        self.selected_transfers = []
-        self.selected_users = []
+        self.selected_transfers = set()
+        self.selected_users = set()
 
         self.widget.get_selection().selected_foreach(self.SelectedTransfersCallback)
 
@@ -175,7 +178,6 @@ class TransferList:
             self.frame.BanUser(user)
 
     def OnFileSearch(self, widget):
-        self.select_transfers()
 
         for transfer in self.selected_transfers:
             self.frame.SearchEntry.set_text(transfer.filename.rsplit("\\", 1)[1])
@@ -191,137 +193,111 @@ class TransferList:
 
     def SelectedTransfersCallback(self, model, path, iter):
 
+        self.SelectTransfer(model, iter, selectuser=True)
+
+        # If we're in grouping mode, select any transfers under the selected
+        # user or folder
+        self.SelectChildTransfers(model, model.iter_children(iter))
+
+    def SelectChildTransfers(self, model, iter):
+
+        while iter is not None:
+            self.SelectTransfer(model, iter)
+            self.SelectChildTransfers(model, model.iter_children(iter))
+            iter = model.iter_next(iter)
+
+    def SelectTransfer(self, model, iter, selectuser=False):
         user = model.get_value(iter, 0)
         file = model.get_value(iter, 10)
 
         for i in self.list:
             if i.user == user and i.filename == file:
-                self.selected_transfers.append(i)
+                self.selected_transfers.add(i)
                 break
 
-        if user not in self.selected_users:
-            self.selected_users.append(user)
+        if selectuser:
+            self.selected_users.add(user)
 
     def TranslateStatus(self, status):
 
-        if status == "Waiting for download":
-            newstatus = _("Waiting for download")
-        elif status == "Waiting for upload":
-            newstatus = _("Waiting for upload")
-        elif status == "Requesting file":
-            newstatus = _("Requesting file")
-        elif status == "Initializing transfer":
-            newstatus = _("Initializing transfer")
-        elif status == "Cannot connect":
-            newstatus = _("Cannot connect")
-        elif status == "Waiting for peer to connect":
-            newstatus = _("Waiting for peer to connect")
-        elif status == "Connecting":
-            newstatus = _("Connecting")
-        elif status == "Getting address":
-            newstatus = _("Getting address")
-        elif status == "Getting status":
-            newstatus = _("Getting status")
-        elif status == "Queued":
-            newstatus = _("Queued")
-        elif status == "User logged off":
-            newstatus = _("User logged off")
-        elif status == "Aborted":
-            newstatus = _("Aborted")
-        elif status == "Finished":
-            newstatus = _("Finished")
-        elif status == "Paused":
-            newstatus = _("Paused")
-        elif status == "Transferring":
-            newstatus = _("Transferring")
-        elif status == "Filtered":
-            newstatus = _("Filtered")
-        elif status == "Connection closed by peer":
-            newstatus = _("Connection closed by peer")
-        elif status == "File not shared":
-            newstatus = _("File not shared")
-        elif status == "Establishing connection":
-            newstatus = _("Establishing connection")
-        elif status == "Download directory error":
-            newstatus = _("Download directory error")
-        elif status == "Local file error":
-            newstatus = _("Local file error")
-        elif status == "Remote file error":
-            newstatus = _("Remote file error")
-        else:
+        try:
+            newstatus = self.statuses[status]
+        except KeyError:
             newstatus = status
 
         return newstatus
 
-    def finalupdate(self, func):
+    def update(self, transfer=None, forceupdate=False):
 
-        now = time()
+        curtime = time()
 
-        # I had a logical explanation about why it has to be 3*delay, but I
-        # forgot. Something to do with the timeout being 2*delay
-        if (now - self.lastupdate) < (3 * self.MINIMUM_GUI_DELAY):
-            # The list has been updated recently,
-            # trying again later.
-            return True
+        if (curtime - self.last_save) > 15:
 
-        self.update(forced=True)  # delayed updates can never trigger a new timer
-        self.finalupdatetimerid = None
+            """ Save downloads list to file every 15 seconds """
 
-        return False  # Stopping timeout
+            if self.frame.np.transfers is not None:
+                self.frame.np.transfers.SaveDownloads()
 
-    def update(self, transfer=None, forced=False, nochildupdate=False):
+            self.last_save = curtime
 
-        current_page = self.frame.MainNotebook.get_current_page()
-        my_page = self.frame.MainNotebook.page_num(self.myvbox)
+        transferring = (transfer is not None and transfer.status == "Transferring")
 
-        if (current_page == my_page):
-            self._update(transfer, forced, nochildupdate)
+        if not forceupdate and \
+            transferring and \
+                (curtime - self.last_ui_update) <= 1:
 
-    def _update(self, transfer=None, forced=True, nochildupdate=False):
+            """ We save CPU by not updating the transfer list every time a part of
+            a file is transferred """
 
-        now = time()
+            return
 
-        if forced:
-            self.lastupdate = time()  # ...we're working...
+        if not forceupdate:
+            current_page = self.frame.MainNotebook.get_current_page()
+            my_page = self.frame.MainNotebook.page_num(self.myvbox)
+
+            if (current_page != my_page):
+                """ No need to do unnecessary work if transfers are not visible """
+                return
 
         if transfer is not None:
             self.update_specific(transfer)
-        elif not nochildupdate and self.list is not None:
+        elif self.list is not None:
 
             for i in self.list:
                 self.update_specific(i)
 
-        self.frame.UpdateBandwidth()
+        if transferring:
+            finished = False
+        else:
+            finished = (transfer is not None and transfer.status == "Finished")
+
+        if not forceupdate and \
+            not finished and \
+                (curtime - self.last_ui_update) <= 1:
+
+            """ Unless a transfer finishes, use a cooldown to avoid updating
+            too often """
 
-        # The rest is just summarizing so it's not too important.
-        # It's fairly CPU intensive though, so we only do it if we haven't updated it recently
-        if not forced and (now - self.lastupdate) < self.MINIMUM_GUI_DELAY:
-            if not self.finalupdatetimerid:
-                self.finalupdatetimerid = True  # I'm not sure if gobject returns fast enough
-                self.finalupdatetimerid = GLib.timeout_add(self.MINIMUM_GUI_DELAY_SLEEP, self.finalupdate, self.update)
             return
 
-        self.lastupdate = time()  # ...we're working...
+        self.update_parent_rows()
 
-        # Save downloads to file
-        if self.frame.np.transfers is not None:
-            self.frame.np.transfers.SaveDownloads()
+    def update_parent_rows(self, only_remove=False):
 
         # Remove empty parent rows
-        for (path, pathiter) in [x for x in self.paths.items()]:
+        for (path, pathiter) in list(self.paths.items()):
             if not self.transfersmodel.iter_has_child(pathiter):
                 self.transfersmodel.remove(pathiter)
                 del self.paths[path]
-            else:
+            elif not only_remove:
                 self.update_parent_row(pathiter)
 
-        for (username, useriter) in [x for x in self.users.items()]:
+        for (username, useriter) in list(self.users.items()):
             if useriter != 0:
                 if not self.transfersmodel.iter_has_child(useriter):
                     self.transfersmodel.remove(useriter)
                     del self.users[username]
-                    self.frame.UpdateBandwidth()
-                else:
+                elif not only_remove:
                     self.update_parent_row(useriter)
             else:
                 # No grouping
@@ -331,12 +307,12 @@ class TransferList:
                         break
                 else:
                     del self.users[username]
-                    self.frame.UpdateBandwidth()
 
-        self.lastupdate = time()  # ...and we're done
+        self.frame.UpdateBandwidth()
+        self.last_ui_update = time()
 
     def update_parent_row(self, initer):
-        files = self.transfersmodel.iter_n_children(initer)
+
         speed = 0.0
         percent = totalsize = position = 0
         hspeed = helapsed = left = ""
@@ -345,9 +321,10 @@ class TransferList:
         salientstatus = ""
         extensions = {}
 
-        for f in range(files):
+        iter = self.transfersmodel.iter_children(initer)
+
+        while iter is not None:
 
-            iter = self.transfersmodel.iter_nth_child(initer, f)
             status = self.transfersmodel.get_value(iter, 11)
 
             if salientstatus in ('', "Finished", "Filtered"):  # we prefer anything over ''/finished
@@ -365,6 +342,7 @@ class TransferList:
 
             if status == "Filtered":
                 # We don't want to count filtered files when calculating the progress
+                iter = self.transfersmodel.iter_next(iter)
                 continue
 
             elapsed += self.transfersmodel.get_value(iter, 15)
@@ -379,6 +357,8 @@ class TransferList:
             if status in ("Transferring", "Banned", "Getting address", "Establishing connection"):
                 salientstatus = status
 
+            iter = self.transfersmodel.iter_next(iter)
+
         if totalsize > 0:
             percent = ((100 * position) / totalsize)
 
@@ -392,7 +372,7 @@ class TransferList:
         if len(extensions) == 0:
             extensions = ""
         elif len(extensions) == 1:
-            extensions = " (" + _("All %(ext)s") % {'ext': list(extensions.keys())[0]} + ")"
+            extensions = " (" + self.extension_list_template % {'ext': list(extensions.keys())[0]} + ")"
         else:
             extensionlst = [(extensions[key], key) for key in extensions]
             extensionlst.sort(reverse=True)
@@ -400,7 +380,7 @@ class TransferList:
 
         self.transfersmodel.set(
             initer,
-            2, _("%(number)2s files ") % {'number': filecount} + extensions,
+            2, self.files_template % {'number': filecount} + extensions,
             3, self.TranslateStatus(salientstatus),
             5, percent,
             6, "%s / %s" % (HumanSize(position), HumanSize(totalsize)),
@@ -433,6 +413,8 @@ class TransferList:
 
         try:
             size = int(transfer.size)
+            if size < 0 or size > maxsize:
+                size = 0
         except TypeError:
             size = 0
 
@@ -471,7 +453,7 @@ class TransferList:
                 # Group by folder, path not visible
                 path = None
             else:
-                path = transfer.path
+                path = '/'.join(reversed(transfer.path.split('/')))
 
             self.transfersmodel.set(
                 transfer.iter,
@@ -510,15 +492,17 @@ class TransferList:
 
                     """ Paths can be empty if files are downloaded individually, make sure we
                     don't add files to the wrong user in the TreeView """
-                    path = user + transfer.path
+                    path = transfer.path
+                    user_path = user + path
+                    reverse_path = '/'.join(reversed(path.split('/')))
 
-                    if path not in self.paths:
-                        self.paths[path] = self.transfersmodel.append(
+                    if user_path not in self.paths:
+                        self.paths[user_path] = self.transfersmodel.append(
                             self.users[user],
-                            [user, transfer.path, "", "", "", 0, "", "", "", "", "", "", 0, 0, 0, 0, filecount, 0]
+                            [user, reverse_path, "", "", "", 0, "", "", "", "", "", "", 0, 0, 0, 0, filecount, 0]
                         )
 
-                    parent = self.paths[path]
+                    parent = self.paths[user_path]
             else:
                 # No grouping
 
@@ -533,7 +517,7 @@ class TransferList:
                 # Group by folder, path not visible
                 path = None
             else:
-                path = transfer.path
+                path = '/'.join(reversed(transfer.path.split('/')))
 
             iter = self.transfersmodel.append(
                 parent,
@@ -553,13 +537,13 @@ class TransferList:
         if transfer.iter is not None:
             self.transfersmodel.remove(transfer.iter)
 
-        self.update(nochildupdate=True)
+        self.update_parent_rows(only_remove=True)
 
     def Clear(self):
         self.users.clear()
         self.paths.clear()
-        self.selected_transfers = []
-        self.selected_users = []
+        self.selected_transfers = set()
+        self.selected_users = set()
         self.transfersmodel.clear()
 
         if self.list is not None:
@@ -568,14 +552,11 @@ class TransferList:
 
     def OnPopupMenuUsers(self, widget):
 
-        self.select_transfers()
-
         self.popup_menu_users.clear()
 
         if len(self.selected_users) > 0:
 
             items = []
-            self.selected_users.sort(key=str.lower)
 
             for user in self.selected_users:
 
@@ -645,13 +626,13 @@ class TransferList:
         self.select_transfers()
 
     def OnCopyURL(self, widget):
-        i = self.selected_transfers[0]
+        i = next(iter(self.selected_transfers))
         self.frame.SetClipboardURL(i.user, i.filename)
 
     def OnCopyDirURL(self, widget):
 
-        i = self.selected_transfers[0]
-        path = "\\".join(i.filename.split("\\")[:-1]) + "\\"
+        i = next(iter(self.selected_transfers))
+        path = "\\".join(i.filename.split("\\")[:-1])
 
         if path[:-1] != "/":
             path += "/"
@@ -660,14 +641,14 @@ class TransferList:
 
     def OnAbortTransfer(self, widget, remove=False, clear=False):
 
-        transfers = self.selected_transfers
-
-        for i in transfers:
+        for i in self.selected_transfers:
 
             if i.status != "Finished":
                 self.frame.np.transfers.AbortTransfer(i, remove)
-                i.status = "Aborted"
-                self.update(i)
+
+                if not clear:
+                    i.status = "Aborted"
+                    self.update(i)
 
             if clear:
                 self.remove_specific(i)
