@@ -23,6 +23,7 @@
 
 import os
 from gettext import gettext as _
+from sys import maxsize
 
 from gi.repository import Gdk
 from gi.repository import GObject as gobject
@@ -31,14 +32,13 @@ from gi.repository import Gtk as gtk
 from _thread import start_new_thread
 from pynicotine import slskmessages
 from pynicotine.gtkgui.dirchooser import ChooseDir
-from pynicotine.gtkgui.entrydialog import input_box
+from pynicotine.gtkgui.dialogs import ComboBoxDialog
+from pynicotine.gtkgui.utils import HideColumns
 from pynicotine.gtkgui.utils import HumanSize
 from pynicotine.gtkgui.utils import InitialiseColumns
 from pynicotine.gtkgui.utils import PopupMenu
-from pynicotine.gtkgui.utils import PressHeader
 from pynicotine.gtkgui.utils import SetTreeviewSelectedRow
 from pynicotine.utils import CleanFile
-from pynicotine.utils import displayTraceback
 from pynicotine.utils import executeCommand
 from pynicotine.utils import GetUserDirectories
 from pynicotine.utils import GetResultBitrateLength
@@ -180,9 +180,9 @@ class UserBrowse:
         cols = InitialiseColumns(
             self.FileTreeView,
             [_("Filename"), widths[0], "text", self.CellDataFunc],
-            [_("Size"), widths[1], "text", self.CellDataFunc],
-            [_("Bitrate"), widths[2], "text", self.CellDataFunc],
-            [_("Length"), widths[3], "text", self.CellDataFunc]
+            [_("Size"), widths[1], "number", self.CellDataFunc],
+            [_("Bitrate"), widths[2], "number", self.CellDataFunc],
+            [_("Length"), widths[3], "number", self.CellDataFunc]
         )
         cols[0].set_sort_column_id(0)
         cols[1].set_sort_column_id(4)
@@ -190,18 +190,7 @@ class UserBrowse:
         cols[3].set_sort_column_id(5)
         self.FileStore.set_sort_column_id(0, gtk.SortType.ASCENDING)
 
-        try:
-            for i in range(len(cols)):
-
-                parent = cols[i].get_widget().get_ancestor(gtk.Button)
-                if parent:
-                    parent.connect("button_press_event", PressHeader)
-
-                # Read Show / Hide column settings from last session
-                cols[i].set_visible(self.frame.np.config.sections["columns"]["userbrowse"][i])
-        except IndexError:
-            # Column count in config is probably incorrect (outdated?), don't crash
-            pass
+        HideColumns(cols, self.frame.np.config.sections["columns"]["userbrowse"])
 
         self.FileTreeView.get_selection().set_mode(gtk.SelectionMode.MULTIPLE)
         self.FileTreeView.set_headers_clickable(True)
@@ -240,18 +229,6 @@ class UserBrowse:
     def OnPopupMenuDummy(self, widget):
         pass
 
-    def Attach(self, widget=None):
-        self.userbrowses.attach_tab(self.Main)
-
-    def Detach(self, widget=None):
-        self.userbrowses.detach_tab(
-            self.Main,
-            _("Nicotine+ User Browse: %(user)s (%(status)s)") % {
-                'user': self.user,
-                'status': [_("Offline"), _("Away"), _("Online")][self.status]
-            }
-        )
-
     def ConnClose(self):
         pass
 
@@ -280,9 +257,7 @@ class UserBrowse:
         return True
 
     def ChangeColours(self):
-        self.frame.SetTextBG(self.FileTreeView)
-        self.frame.SetTextBG(self.FolderTreeView)
-        self.frame.SetTextBG(self.entry4)
+        self.frame.SetTextBG(self.SearchEntry)
 
         self.frame.ChangeListFont(self.FolderTreeView, self.frame.np.config.sections["ui"]["browserfont"])
         self.frame.ChangeListFont(self.FileTreeView, self.frame.np.config.sections["ui"]["browserfont"])
@@ -297,10 +272,10 @@ class UserBrowse:
 
         if self.ExpandButton.get_active():
             self.FolderTreeView.expand_all()
-            self.ExpandDirectoriesImage.set_from_stock(gtk.STOCK_REMOVE, 4)
+            self.expand.set_from_icon_name("list-remove-symbolic", gtk.IconSize.BUTTON)
         else:
             self.FolderTreeView.collapse_all()
-            self.ExpandDirectoriesImage.set_from_stock(gtk.STOCK_ADD, 4)
+            self.expand.set_from_icon_name("list-add-symbolic", gtk.IconSize.BUTTON)
 
             dirs = list(self.directories.keys())
             dirs.sort()
@@ -312,11 +287,16 @@ class UserBrowse:
 
     def OnFolderClicked(self, widget, event):
 
-        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
-            self.OnDownloadDirectory(widget)
-            return True
-        elif event.button == 3:
-            return self.OnFolderPopupMenu(widget, event)
+        pathinfo = widget.get_path_at_pos(event.x, event.y)
+
+        if pathinfo is not None:
+
+            if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
+                self.OnDownloadDirectory(widget)
+                return True
+
+            elif event.button == 3:
+                return self.OnFolderPopupMenu(widget, event)
 
         return False
 
@@ -338,15 +318,22 @@ class UserBrowse:
 
     def OnFileClicked(self, widget, event):
 
-        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
-            self.selected_files = []
-            self.FileTreeView.get_selection().selected_foreach(self.SelectedFilesCallback)
-            self.OnDownloadFiles(widget)
-            self.FileTreeView.get_selection().unselect_all()
-            return True
-        elif event.button == 3:
+        if event.button == 3:
             return self.OnFilePopupMenu(widget, event)
 
+        else:
+            pathinfo = widget.get_path_at_pos(event.x, event.y)
+
+            if pathinfo is None:
+                widget.get_selection().unselect_all()
+
+            elif event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
+                self.selected_files = []
+                self.FileTreeView.get_selection().selected_foreach(self.SelectedFilesCallback)
+                self.OnDownloadFiles(widget)
+                self.FileTreeView.get_selection().unselect_all()
+                return True
+
         return False
 
     def OnFilePopupMenu(self, widget, event):
@@ -396,10 +383,10 @@ class UserBrowse:
         self.totalsize = 0
         for dir, files in self.shares:
             for filedata in files:
-                if filedata[2] < 18446744000000000000:
+                if filedata[2] < maxsize:
                     self.totalsize += filedata[2]
                 else:
-                    print("Unbelievable filesize: %s, %s" % (HumanSize(filedata[2]), repr(filedata)))
+                    pass
 
         self.AmountShared.set_text(_("Shared: %s") % HumanSize(self.totalsize))
         self.NumDirectories.set_text(_("Dirs: %s") % len(self.shares))
@@ -429,6 +416,7 @@ class UserBrowse:
         dirseparator = '\\'
 
         # If there is no share
+
         if self.shares == []:
 
             # Set the model of the treeviex
@@ -465,7 +453,7 @@ class UserBrowse:
             """
 
             # Foreach subdir
-            for subdir in list(dictdir.keys()):
+            for subdir in dictdir:
 
                 if parent is None:
                     # The first sudirs are attached to the root (None)
@@ -529,8 +517,13 @@ class UserBrowse:
         for file in files:
             # Filename, HSize, Bitrate, HLength, Size, Length, RawFilename
             rl = 0
+
             try:
                 size = int(file[2])
+
+                # Some clients send incorrect file sizes
+                if size < 0 or size > maxsize:
+                    size = 0
             except ValueError:
                 size = 0
 
@@ -543,8 +536,9 @@ class UserBrowse:
 
             try:
                 self.files[f[0]] = self.FileStore.append(f)
-            except Exception:
-                displayTraceback()
+            except Exception as msg:
+                error = _("Error while attempting to display folder '%(folder)s', reported error: %(error)s" % {'folder': directory, 'error': msg})
+                self.frame.logMessage(error)
 
     def OnSave(self, widget):
         sharesdir = os.path.join(self.data_dir, "usershares")
@@ -674,6 +668,9 @@ class UserBrowse:
 
         ldir = prefix + dir.split("\\")[-1]
 
+        # Check if folder already exists on system
+        ldir = self.frame.np.transfers.FolderDestination(self.user, ldir)
+
         for d, f in self.shares:
 
             # Find the wanted directory
@@ -707,7 +704,15 @@ class UserBrowse:
                 size = file[2]
                 h_bitrate, bitrate, h_length = GetResultBitrateLength(size, file[4])
 
-                self.frame.np.transfers.getFile(self.user, path, ldir, size=size, bitrate=h_bitrate, length=h_length, checkduplicate=True)
+                self.frame.np.transfers.getFile(
+                    self.user,
+                    path,
+                    ldir,
+                    size=size,
+                    bitrate=h_bitrate,
+                    length=h_length,
+                    checkduplicate=True
+                )
 
         if not recurse:
             return
@@ -776,9 +781,9 @@ class UserBrowse:
             users.append(entry[0])
 
         users.sort()
-        user = input_box(
-            self.frame,
-            title=_("Nicotine+: Upload Directory's Contents"),
+        user = ComboBoxDialog(
+            parent=self.frame.MainWindow,
+            title=_("Upload Directory's Contents"),
             message=_('Enter the User you wish to upload to:'),
             droplist=users
         )
@@ -832,9 +837,9 @@ class UserBrowse:
             users.append(entry[0])
 
         users.sort()
-        user = input_box(
-            self.frame,
-            title=_('Nicotine+: Upload File(s)'),
+        user = ComboBoxDialog(
+            parent=self.frame.MainWindow,
+            title=_('Upload File(s)'),
             message=_('Enter the User you wish to upload to:'),
             droplist=users
         )
@@ -881,7 +886,7 @@ class UserBrowse:
 
     def OnSearch(self, widget):
 
-        query = widget.get_text().lower()
+        query = self.SearchEntry.get_text().lower()
 
         if self.query == query:
             self.search_position += 1
@@ -933,11 +938,8 @@ class UserBrowse:
         del self.userbrowses.users[self.user]
         self.frame.np.ClosePeerConnection(self.conn)
 
-        if self.userbrowses.is_tab_detached(self.Main):
-            self.Main.get_parent_window().destroy()
-        else:
-            self.userbrowses.remove_page(self.Main)
-            self.Main.destroy()
+        self.userbrowses.remove_page(self.Main)
+        self.Main.destroy()
 
     def OnRefresh(self, widget):
         self.FolderTreeView.set_sensitive(False)
