@@ -55,7 +55,7 @@ class UserTabs(IconNotebook):
             self.frame.images,
             angle=ui["labelinfo"],
             tabclosers=ui["tabclosers"],
-            show_image=ui["tab_icons"],
+            show_image=self.frame.np.config.sections["notifications"]["notification_tab_icons"],
             reorderable=ui["tab_reorderable"],
             show_status_image=ui["tab_status_icons"],
             notebookraw=notebookraw
@@ -160,7 +160,6 @@ class UserTabs(IconNotebook):
             ("$" + _("Ban this user"), popup.OnBanUser),
             ("$" + _("Ignore this user"), popup.OnIgnoreUser),
             ("", None),
-            ("#" + _("Detach this tab"), self.users[user].Detach),
             ("#" + _("Close this tab"), self.users[user].OnClose)
         )
 
@@ -253,10 +252,10 @@ class UserInfo:
         cols[0].set_sort_column_id(0)
         self.likesStore.set_sort_column_id(0, gtk.SortType.ASCENDING)
 
-        self.tag_local = self.makecolour("chatremote")  # noqa: F821
+        self.tag_local = self.makecolour("chatremote")
         self.ChangeColours()
 
-        self.likes_popup_menu = popup = PopupMenu(self)
+        self.likes_popup_menu = popup = PopupMenu(self.frame)
         popup.setup(
             ("$" + _("I _like this"), self.frame.OnLikeRecommendation),
             ("$" + _("I _don't like this"), self.frame.OnDislikeRecommendation),
@@ -266,7 +265,7 @@ class UserInfo:
 
         self.Likes.connect("button_press_event", self.OnPopupLikesMenu)
 
-        self.hates_popup_menu = popup = PopupMenu(self)
+        self.hates_popup_menu = popup = PopupMenu(self.frame)
         popup.setup(
             ("$" + _("I _like this"), self.frame.OnLikeRecommendation),
             ("$" + _("I _don't like this"), self.frame.OnDislikeRecommendation),
@@ -276,7 +275,7 @@ class UserInfo:
 
         self.Hates.connect("button_press_event", self.OnPopupHatesMenu)
 
-        self.image_menu = popup = PopupMenu(self)
+        self.image_menu = popup = PopupMenu(self.frame)
         popup.setup(
             ("#" + _("Zoom 1:1"), self.MakeZoomNormal),
             ("#" + _("Zoom In"), self.MakeZoomIn),
@@ -332,18 +331,6 @@ class UserInfo:
     def ConnClose(self):
         pass
 
-    def Attach(self, widget=None):
-        self.userinfos.attach_tab(self.Main)
-
-    def Detach(self, widget=None):
-        self.userinfos.detach_tab(
-            self.Main,
-            _("Nicotine+ Userinfo: %(user)s (%(status)s)") % {
-                'user': self.user,
-                'status': [_("Offline"), _("Away"), _("Online")][self.status]
-            }
-        )
-
     def CellDataFunc(self, column, cellrenderer, model, iter, dummy="dummy"):
 
         colour = self.frame.np.config.sections["ui"]["search"]
@@ -388,10 +375,6 @@ class UserInfo:
 
         self.changecolour(self.tag_local, "chatremote")
 
-        self.frame.SetTextBG(self.descr)
-        self.frame.SetTextBG(self.Likes)
-        self.frame.SetTextBG(self.Hates)
-
         self.frame.ChangeListFont(self.Likes, self.frame.np.config.sections["ui"]["listfont"])
         self.frame.ChangeListFont(self.Hates, self.frame.np.config.sections["ui"]["listfont"])
 
@@ -450,7 +433,7 @@ class UserInfo:
                 gc.collect()
                 self.actual_zoom = 0
                 self.SavePicture.set_sensitive(True)
-            except TypeError as e:  # noqa: F841
+            except TypeError:
                 name = tempfile.mktemp()
                 f = open(name, "w")
                 f.write(pic)
@@ -502,11 +485,8 @@ class UserInfo:
         del self.userinfos.users[self.user]
         self.frame.np.ClosePeerConnection(self.conn)
 
-        if self.userinfos.is_tab_detached(self.Main):
-            self.Main.get_parent_window().destroy()
-        else:
-            self.userinfos.remove_page(self.Main)
-            self.Main.destroy()
+        self.userinfos.remove_page(self.Main)
+        self.Main.destroy()
 
     def OnSavePicture(self, widget):
 
