@@ -21,16 +21,13 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-import locale
-import math
+
 import os
 import re
 import sys
 import time
 import types
-import urllib.error
 import urllib.parse
-import urllib.request
 import webbrowser
 from gettext import gettext as _
 
@@ -41,13 +38,15 @@ from gi.repository import Gtk as gtk
 from gi.repository import Pango as pango
 
 from pynicotine import slskmessages
+from pynicotine.gtkgui.dialogs import EntryDialog
 from pynicotine.gtkgui.countrycodes import code2name
+from pynicotine.utils import CleanFile
 from pynicotine.utils import executeCommand
 
 
 DECIMALSEP = ""
 
-URL_RE = re.compile("(\\w+\\://.+?)[\\s\\(\\)]|(www\\.\\w+\\.\\w+.*?)[\\s\\(\\)]|(mailto\\:\\w.+?)[\\s\\(\\)]")
+URL_RE = re.compile("(\\w+\\://[^\\s]+)|(www\\.\\w+\\.\\w+.*?)|(mailto\\:[^\\s]+)")
 PROTOCOL_HANDLERS = {}
 CATCH_URLS = 0
 HUMANIZE_URLS = 0
@@ -55,38 +54,6 @@ USERNAMEHOTSPOTS = 0
 NICOTINE = None
 
 
-def popupWarning(parent, title, warning, icon=None):
-    dlg = gtk.Dialog(title=title, parent=parent, buttons=(gtk.STOCK_OK, gtk.ResponseType.OK))
-    dlg.set_default_response(gtk.ResponseType.OK)
-    dlg.set_icon(icon)
-    dlg.set_border_width(10)
-    dlg.vbox.set_spacing(10)
-    hbox = gtk.HBox(spacing=5)
-    hbox.set_border_width(5)
-    hbox.show()
-    dlg.vbox.pack_start(hbox, True, True, 0)
-
-    image = gtk.Image()
-    image.set_padding(0, 0)
-    icon = gtk.STOCK_DIALOG_WARNING
-    image.set_from_stock(icon, 4)
-    image.show()
-
-    hbox.pack_start(image, True, True, 0)
-    label = gtk.Label()
-    label.set_markup(warning)
-    label.set_line_wrap(True)
-    hbox.pack_start(label, True, True, 0)
-
-    dlg.vbox.show_all()
-
-    result = None  # noqa: F841
-    if dlg.run() == gtk.ResponseType.OK:
-        dlg.destroy()
-
-    return 0
-
-
 # we could move this into a new class
 previouscountrypath = None
 
@@ -146,20 +113,6 @@ def showCountryTooltip(widget, x, y, tooltip, sourcecolumn, stripprefix='flag_')
     return True
 
 
-def recode(s):
-    try:
-        return s.decode(locale.nl_langinfo(locale.CODESET), "replace").encode("utf-8", "replace")
-    except Exception:
-        return s
-
-
-def recode2(s):
-    try:
-        return s.decode("utf-8", "replace").encode(locale.nl_langinfo(locale.CODESET), "replace")
-    except Exception:
-        return s
-
-
 def FillFileGroupingCombobox(combobox):
     grouplist = gtk.ListStore(str)
     groups = [
@@ -215,20 +168,28 @@ def InitialiseColumns(treeview, *args):
 
         if c[2] == "text":
             renderer = gtk.CellRendererText()
+            renderer.set_padding(10, 3)
+
+            column = gtk.TreeViewColumn(c[0], renderer, text=i)
+        elif c[2] == "center":
+            renderer = gtk.CellRendererText()
+            renderer.set_property("xalign", 0.5)
+
             column = gtk.TreeViewColumn(c[0], renderer, text=i)
         elif c[2] == "number":
             renderer = gtk.CellRendererText()
+            renderer.set_property("xalign", 0.9)
+
             column = gtk.TreeViewColumn(c[0], renderer, text=i)
-            renderer.set_property("xalign", 1)
-        elif c[2] == "colored":
-            renderer = gtk.CellRendererText()
-            column = gtk.TreeViewColumn(c[0], renderer, text=i, foreground=c[3][0], background=c[3][1])
+            column.set_alignment(0.9)
         elif c[2] == "edit":
             renderer = gtk.CellRendererText()
+            renderer.set_padding(10, 3)
             renderer.set_property('editable', True)
             column = gtk.TreeViewColumn(c[0], renderer, text=i)
         elif c[2] == "combo":
             renderer = gtk.CellRendererCombo()
+            renderer.set_padding(10, 3)
             renderer.set_property('text-column', 0)
             renderer.set_property('editable', True)
             column = gtk.TreeViewColumn(c[0], renderer, text=i)
@@ -258,8 +219,22 @@ def InitialiseColumns(treeview, *args):
         if len(c) > 3 and type(c[3]) is not list:
             column.set_cell_data_func(renderer, c[3])
 
+        if len(c) > 4:
+            foreground = c[4][0]
+            background = c[4][1]
+
+            if foreground == "":
+                foreground = None
+
+            if background == "":
+                background = None
+
+            renderer.set_property("foreground", foreground)
+            renderer.set_property("background", background)
+
         column.set_reorderable(False)
         column.set_widget(gtk.Label.new(c[0]))
+        column.get_widget().set_margin_start(6)
         column.get_widget().show()
 
         treeview.append_column(column)
@@ -271,6 +246,32 @@ def InitialiseColumns(treeview, *args):
     return cols
 
 
+def HideColumns(cols, visibility_list):
+    try:
+        for i in range(len(cols)):
+
+            parent = cols[i].get_widget().get_ancestor(gtk.Button)
+            if parent:
+                parent.connect('button_press_event', PressHeader)
+
+            # Read Show / Hide column settings from last session
+            cols[i].set_visible(visibility_list[i])
+
+        # Make sure the width of the last visible column isn't fixed
+        for i in reversed(range(len(cols))):
+
+            if cols[i].get_visible():
+                column = cols[i]
+                column.set_sizing(gtk.TreeViewColumnSizing.AUTOSIZE)
+                column.set_resizable(False)
+                column.set_fixed_width(-1)
+                break
+
+    except IndexError:
+        # Column count in config is probably incorrect (outdated?), don't crash
+        pass
+
+
 def PressHeader(widget, event):
     if event.button != 3:
         return False
@@ -290,7 +291,7 @@ def PressHeader(widget, event):
                 item.set_sensitive(False)
         else:
             item.set_active(False)
-        item.connect('activate', header_toggle, column)
+        item.connect('activate', header_toggle, columns, pos - 1)
         menu.append(item)
         pos += 1
 
@@ -301,8 +302,29 @@ def PressHeader(widget, event):
     return True
 
 
-def header_toggle(menuitem, column):
+def header_toggle(menuitem, columns, index):
+    column = columns[index]
     column.set_visible(not column.get_visible())
+
+    # Make sure the width of the last visible column isn't fixed
+    for i in reversed(range(len(columns))):
+
+        if columns[i].get_visible():
+            column = columns[i]
+            column.set_sizing(gtk.TreeViewColumnSizing.AUTOSIZE)
+            column.set_resizable(False)
+            column.set_fixed_width(-1)
+            break
+
+    """ If the column we toggled the visibility of is now the last visible one,
+    the previously last visible column should've resized to fit properly now,
+    since it was set to AUTOSIZE. We can now set the previous column to FIXED,
+    and make it resizable again. """
+
+    prev_column = columns[index - 1]
+    prev_column.set_sizing(gtk.TreeViewColumnSizing.FIXED)
+    prev_column.set_resizable(True)
+
     NICOTINE.SaveColumns()
 
 
@@ -310,17 +332,18 @@ def SetTreeviewSelectedRow(treeview, event):
     """Handles row selection when right-clicking in a treeview"""
 
     pathinfo = treeview.get_path_at_pos(event.x, event.y)
+    selection = treeview.get_selection()
 
     if pathinfo is not None:
         path, col, cell_x, cell_y = pathinfo
 
-        selection = treeview.get_selection()
-
         # Make sure we don't attempt to select a single row if the row is already
         # in a selection of multiple rows, otherwise the other rows will be unselected
         if selection.count_selected_rows() <= 1 or not selection.path_is_selected(path):
             treeview.grab_focus()
             treeview.set_cursor(path, col, False)
+    else:
+        selection.unselect_all()
 
 
 def ScrollBottom(widget):
@@ -413,34 +436,38 @@ def AppendLine(textview, line, tag=None, timestamp=None, showstamp=True, timesta
             _append(buffer, section, tag)
 
     scrolledwindow = textview.get_parent()
-    va = scrolledwindow.get_vadjustment()
+
     try:
-        bottom = va.value >= (va.upper - int(va.page_size * 1.5))
+        va = scrolledwindow.get_vadjustment()
     except AttributeError:
-        bottom = True
+        # scrolledwindow may have disappeared already while Nicotine+ was shutting down
+        return
+
+    bottom = (va.get_value() + va.get_page_size()) >= va.get_upper()
 
     buffer = textview.get_buffer()
+    text_iter_start, text_iter_end = buffer.get_bounds()
     linenr = buffer.get_line_count()
-    ME = 0
-
-    if line.startswith("* "):
-        ME = 1  # noqa: F841
 
     TIMESTAMP = None
     TS = 0
 
-    if NICOTINE.np.config.sections["logging"]["timestamps"] and showstamp:
+    if showstamp and NICOTINE.np.config.sections["logging"]["timestamps"]:
         if timestamp_format and not timestamp:
-            TIMESTAMP = recode(time.strftime(timestamp_format))
-            line = "%s %s\n" % (TIMESTAMP, line)
+            TIMESTAMP = time.strftime(timestamp_format)
+            line = "%s %s" % (TIMESTAMP, line)
         elif timestamp_format and timestamp:
-            TIMESTAMP = recode(time.strftime(timestamp_format, time.localtime(timestamp)))
-            line = "%s %s\n" % (TIMESTAMP, line)
-    else:
-        line += "\n"
+            TIMESTAMP = time.strftime(timestamp_format, time.localtime(timestamp))
+            line = "%s %s" % (TIMESTAMP, line)
+
+    # Ensure newlines are in the correct place
+    # We want them before the content, to prevent adding an empty line at the end of the TextView
+    line = line.strip("\n")
+    if text_iter_end.get_offset() > 0:
+        line = "\n" + line
 
     if TIMESTAMP is not None:
-        TS = len(TIMESTAMP)
+        TS = len("\n") + len(TIMESTAMP)
 
     # Append timestamp, if one exists, cut it from remaining line (to avoid matching against username)
     _append(buffer, line[:TS], tag)
@@ -451,12 +478,12 @@ def AppendLine(textview, line, tag=None, timestamp=None, showstamp=True, timesta
     while CATCH_URLS and match:
         start = line[:match.start()]
         _usertag(buffer, start)
-        url = match.group()[:-1]
+        url = match.group()
         urltag = _makeurltag(buffer, tag, url)
-        line = line[match.end() - 1:]
+        line = line[match.end():]
 
         if url.startswith("slsk://") and HUMANIZE_URLS:
-            url = urllib.request.url2pathname(url)
+            url = urllib.parse.unquote(url)
 
         _append(buffer, url, urltag)
         # Match remaining url
@@ -516,11 +543,11 @@ class BuddiesComboBox:
             del self.items[item]
 
 
-class ImageLabel(gtk.HBox):
+class ImageLabel(gtk.Box):
 
     def __init__(self, label="", image=None, onclose=None, closebutton=False, angle=0, show_image=True, statusimage=None, show_status_image=False):
 
-        gtk.HBox.__init__(self)
+        gtk.Box.__init__(self)
 
         self.closebutton = closebutton
         self.angle = angle
@@ -536,16 +563,9 @@ class ImageLabel(gtk.HBox):
         self.statusimage = gtk.Image()
         self.label = gtk.Label()
         self.text = label
-        if NICOTINE.np.config.sections["ui"]["tab_colors"]:
-            color = NICOTINE.np.config.sections["ui"]["tab_default"]
-        else:
-            color = ""
-        if not color:
-            self.label.set_text("%s" % self.text)
-        else:
-            self.label.set_markup("<span foreground=\"%s\">%s</span>" % (color, self.text.replace("<", "&lt;").replace(">", "&gt;").replace("&", "&amp;")))
 
-        self.label.set_alignment(0.0, 0.50)
+        self.set_text(self.text)
+
         self.label.set_angle(angle)
         self.label.show()
 
@@ -571,11 +591,12 @@ class ImageLabel(gtk.HBox):
             self.Box.destroy()
             del self.Box
 
+        self.Box = gtk.Box()
+
         if self.angle in (90, -90):
-            self.Box = gtk.VBox()
+            self.Box.set_orientation(gtk.Orientation.VERTICAL)
         else:
             self.angle = 0
-            self.Box = gtk.HBox()
 
         self.Box.set_spacing(2)
         self.add(self.Box)
@@ -637,7 +658,7 @@ class ImageLabel(gtk.HBox):
             return
         self.button = gtk.Button()
         img = gtk.Image()
-        img.set_from_stock(gtk.STOCK_CLOSE, gtk.IconSize.MENU)
+        img.set_from_icon_name("window-close-symbolic", gtk.IconSize.MENU)
         self.button.add(img)
         if self.onclose is not None:
             self.button.connect("clicked", self.onclose)
@@ -659,19 +680,18 @@ class ImageLabel(gtk.HBox):
         else:
             self.notify = notify
 
-        if NICOTINE.np.config.sections["ui"]["tab_colors"]:
+        color = NICOTINE.np.config.sections["ui"]["tab_default"]
+
+        if NICOTINE.np.config.sections["notifications"]["notification_tab_colors"]:
             if notify == 1:
                 color = NICOTINE.np.config.sections["ui"]["tab_changed"]
             elif notify == 2:
                 color = NICOTINE.np.config.sections["ui"]["tab_hilite"]
-            else:
-                color = NICOTINE.np.config.sections["ui"]["tab_default"]
 
-            try:
-                Gdk.color_parse(color)
-            except Exception:
-                color = ""
-        else:
+        try:
+            rgba = Gdk.RGBA()
+            rgba.parse(color)
+        except Exception:
             color = ""
 
         if text is not None:
@@ -704,11 +724,7 @@ class ImageLabel(gtk.HBox):
         return self.status_img
 
     def set_text(self, lbl):
-        if NICOTINE.np.config.sections["ui"]["tab_colors"]:
-            self.set_text_color(notify=None, text=lbl)
-        else:
-            self.text = lbl
-            self.label.set_text(lbl)
+        self.set_text_color(notify=None, text=lbl)
 
     def get_text(self):
         return self.label.get_text()
@@ -719,7 +735,6 @@ class IconNotebook:
     On top of what a gtk.Notebook provides:
     - You can have icons on the notebook tab.
     - You can choose the label orientation (angle).
-    - You can attach/detach a tab.
     """
 
     def __init__(self, images, angle=0, tabclosers=False, show_image=True, reorderable=True, show_status_image=False, notebookraw=None):
@@ -736,7 +751,6 @@ class IconNotebook:
         self._show_status_image = show_status_image
 
         self.pages = []
-        self.detached_tabs = []
 
         self.Notebook.connect("switch-page", self.dismiss_icon)
         self.Notebook.connect("key_press_event", self.OnKeyPress)
@@ -850,142 +864,8 @@ class IconNotebook:
         if self.Notebook.get_n_pages() == 0:
             self.Notebook.set_show_tabs(False)
 
-    def OnTabWindowDestroy(self, widget, page):
-
-        if self.is_tab_detached(page):
-            self.attach_tab(page, destroying=True)
-
-    def OnFocusIn(self, widget, event):
-        for item in self.detached_tabs:
-            if item[2] == widget:
-                item[3] = True
-                self.OnFocused(item)
-
-    def OnFocusOut(self, widget, event):
-        for item in self.detached_tabs:
-            if item[2] == widget:
-                item[3] = False
-
     def OnFocused(self, item):
-        (page, label, window, focused) = item
         self.frame.Notifications.ClearPage(self, item)
-        self.set_detached_icon(page, 0)
-
-    def attach_tab(self, page, destroying=False):
-
-        pagewidget = label_tab = label_tab_menu = label = status = None
-
-        for item in self.detached_tabs:
-            if item[0] is page:
-                label = item[1]
-                window = item[2]
-                break
-
-        if label is None or window is None:
-            return
-
-        for i in self.pages[:]:
-            if i[0] == page:
-                label = i[1].label.get_text()
-                pagewidget, label_tab, status, label_tab_menu = i
-                break
-
-        for i in (pagewidget, label_tab, label_tab_menu, label, status):
-            if i is None:
-                return
-
-        window.get_child().remove(pagewidget)
-
-        eventbox = gtk.EventBox()
-        eventbox.set_visible_window(False)
-
-        label_tab.show()
-
-        eventbox.add(label_tab)
-        eventbox.show()
-        eventbox.set_events(Gdk.EventType.BUTTON_PRESS_MASK)
-        eventbox.connect('button_press_event', self.on_tab_click, page)
-
-        gtk.Notebook.append_page_menu(self.Notebook, pagewidget, eventbox, label_tab_menu)
-
-        self.Notebook.set_tab_reorderable(page, self.reorderable)
-        self.detached_tabs.remove(item)
-
-        if not destroying:
-            window.destroy()
-
-    def detach_tab(self, page, title=_("Nicotine+")):
-
-        label = None
-        if self.is_tab_detached(page):
-            return
-
-        for i in self.pages[:]:
-            if i[0] == page:
-                pagewidget, label_tab, status, label_tab_menu = i
-                label = label_tab.label.get_text()
-                label_tab.get_parent().remove(label_tab)
-                break
-
-        if label is None:
-            return
-
-        for i in self.detached_tabs:
-            if i[0] == label or i[1] is page:
-                return
-
-        gtk.Notebook.remove_page(self.Notebook, self.page_num(page))
-
-        window = gtk.Window()
-        window.set_title(title)
-        window.set_icon(NICOTINE.images["n"])
-        window.resize(600, 400)
-
-        vbox = gtk.VBox(False, spacing=5)
-        vbox.set_border_width(5)
-        vbox.pack_start(page, True, True, 0)
-        vbox.show()
-
-        window.add(vbox)
-        window.connect("destroy", self.OnTabWindowDestroy, page)
-        window.connect("focus_in_event", self.OnFocusIn)
-        window.connect("focus_out_event", self.OnFocusOut)
-
-        self.detached_tabs.append([page, label, window, False])
-
-        window.show()
-
-    def is_tab_detached(self, page):
-
-        for item in self.detached_tabs:
-            if item[0] is page:
-                return True
-
-        return False
-
-    def is_detached_tab_focused(self, page):
-
-        for item in self.detached_tabs:
-            if item[0] is page:
-                return item[3]
-
-        return False
-
-    def set_detached_icon(self, page, status):
-
-        image = self.images[("n", "hilite3", "hilite")[status]]
-
-        for item in self.detached_tabs:
-            if item[0] is page:
-                window = item[2]
-                window.set_icon(image)
-
-    def set_detached_tab_title(self, page, title):
-
-        for item in self.detached_tabs:
-            if item[0] is page:
-                window = item[2]
-                window.set_title(title)
 
     def on_tab_click(self, widget, event, child):
         pass
@@ -1045,12 +925,6 @@ class IconNotebook:
 
     def request_hilite(self, page):
 
-        if self.is_tab_detached(page):
-            if not self.is_detached_tab_focused(page):
-                self.set_detached_icon(page, 2)
-            return
-
-        # Don't set 'tab' notification icons for detached tabs
         current = self.get_nth_page(self.get_current_page())
         if current == page:
             return
@@ -1060,12 +934,6 @@ class IconNotebook:
 
     def request_changed(self, page):
 
-        if self.is_tab_detached(page):
-            if not self.is_detached_tab_focused(page):
-                self.set_detached_icon(page, 1)
-            return
-
-        # Don't set 'tab' notification icons for detached tabs
         current = self.get_nth_page(self.get_current_page())
         if current == page:
             return
@@ -1218,16 +1086,16 @@ class PopupMenu(gtk.Menu):
         self.frame.BrowseUser(self.user)
 
     def OnPrivateRoomAddUser(self, widget, room):
-        self.frame.PrivateRoomAddUser(room, self.user)
+        self.frame.np.queue.put(slskmessages.PrivateRoomAddUser(room, self.user))
 
     def OnPrivateRoomRemoveUser(self, widget, room):
-        self.frame.PrivateRoomRemoveUser(room, self.user)
+        self.frame.np.queue.put(slskmessages.PrivateRoomRemoveUser(room, self.user))
 
     def OnPrivateRoomAddOperator(self, widget, room):
-        self.frame.PrivateRoomAddOperator(room, self.user)
+        self.frame.np.queue.put(slskmessages.PrivateRoomAddOperator(room, self.user))
 
     def OnPrivateRoomRemoveOperator(self, widget, room):
-        self.frame.PrivateRoomRemoveOperator(room, self.user)
+        self.frame.np.queue.put(slskmessages.PrivateRoomRemoveOperator(room, self.user))
 
     def OnAddToList(self, widget):
 
@@ -1294,7 +1162,7 @@ class PopupMenu(gtk.Menu):
         else:
             days = self.frame.np.privileges_left // 60 // 60 // 24
 
-        text = InputDialog(
+        text = EntryDialog(
             self.frame.MainWindow,
             _("Give privileges") + " " + _("to %(user)s") % {"user": self.user},
             _("Give how many days of global privileges to this user?") + " (" + _("%(days)s days left") % {'days': days} + ")"
@@ -1303,7 +1171,7 @@ class PopupMenu(gtk.Menu):
         if text:
             try:
                 days = int(text)
-                self.frame.GivePrivileges(self.user, days)
+                self.frame.np.queue.put(slskmessages.GivePrivileges(self.user, days))
             except Exception as e:
                 print(e)
 
@@ -1312,7 +1180,6 @@ class PopupMenu(gtk.Menu):
         if self.user is None or self.user == self.frame.np.config.sections["server"]["login"]:
             return False
 
-        user = self.user  # noqa: F841
         items = []
         popup = self.frame.userlist.Popup_Menu_PrivateRooms
         popup.clear()
@@ -1340,82 +1207,51 @@ class PopupMenu(gtk.Menu):
         return True
 
 
-def InputDialog(parent, title, message, default=""):
-
-    dlg = gtk.Dialog(title=title, parent=parent, buttons=(gtk.STOCK_CANCEL, gtk.ResponseType.CANCEL, gtk.STOCK_OK, gtk.ResponseType.OK))
-    dlg.set_default_response(gtk.ResponseType.OK)
-
-    dlg.set_border_width(10)
-    dlg.vbox.set_spacing(10)
-
-    l = gtk.Label(message)  # noqa: E741
-    l.set_alignment(0, 0.5)
-    dlg.vbox.pack_start(l, False, False, 0)
-
-    entry = gtk.Entry()
-    entry.set_activates_default(True)
-    entry.set_text(default)
-    dlg.vbox.pack_start(entry, True, True, 0)
-
-    dlg.vbox.show_all()
-
-    result = None
-    if dlg.run() == gtk.ResponseType.OK:
-        result = entry.get_text()
-
-    dlg.destroy()
-
-    return result
-
-
 def WriteLog(logsdir, fn, msg):
 
     oldumask = os.umask(0o077)
     if not os.path.exists(logsdir):
         os.makedirs(logsdir)
 
-    logfile = open(os.path.join(logsdir, fixpath(fn.replace(os.sep, "-")) + ".log"), 'ab', 0)
+    logfile = open(os.path.join(logsdir, CleanFile(fn.replace(os.sep, "-")) + ".log"), 'ab', 0)
 
     os.umask(oldumask)
 
-    text = "%s %s\n" % (recode(time.strftime(NICOTINE.np.config.sections["logging"]["log_timestamp"])), msg)
+    text = "%s %s\n" % (time.strftime(NICOTINE.np.config.sections["logging"]["log_timestamp"]), msg)
 
     logfile.write(text.encode('UTF-8', 'replace'))
     logfile.flush()
     logfile.close()
 
 
-def fixpath(path):
-    try:
-        if sys.platform == "win32":
-            chars = ["?", "/", "\\", "\"", ":", ">", "<", "|", "*"]
-            for char in chars:
-                path = path.replace(char, "_")
-        return path
-    except Exception:
-        return path
+size_suffixes = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB']
 
 
 def HumanSize(filesize):
     try:
-        if (filesize <= 0):
-            return "0 Bytes"
+        step_unit = 1024.0
 
-        p = int(math.floor(math.log(filesize, 2) / 10))
+        for i in size_suffixes:
+            if filesize < step_unit:
+                return "%3.1f %s" % (filesize, i)
 
-        return "%0.2f %s" % (filesize / math.pow(1024, p), ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'][p])
+            filesize /= step_unit
     except TypeError:
         return filesize
 
 
+speed_suffixes = ['B/s', 'KiB/s', 'MiB/s', 'GiB/s', 'TiB/s', 'PiB/s', 'EiB/s', 'ZiB/s', 'YiB/s']
+
+
 def HumanSpeed(filesize):
     try:
-        if (filesize <= 0):
-            return "0 B/s"
+        step_unit = 1000.0
 
-        p = int(math.floor(math.log(filesize, 2) / 10))
+        for i in speed_suffixes:
+            if filesize < step_unit:
+                return "%3.1f %s" % (filesize, i)
 
-        return "%0.2f %s" % (filesize / math.pow(1024, p), ['B/s', 'KB/s', 'MB/s', 'GB/s', 'TB/s', 'PB/s', 'EB/s', 'ZB/s', 'YB/s'][p])
+            filesize /= step_unit
     except TypeError:
         return filesize
 
