@@ -35,10 +35,10 @@ import logging
 import os
 import queue
 import shutil
-import sys
 import threading
 import time
 from gettext import gettext as _
+from socket import socket
 
 from pynicotine import slskmessages
 from pynicotine import slskproto
@@ -49,7 +49,6 @@ from pynicotine.shares import Shares
 from pynicotine.slskmessages import PopupMessage
 from pynicotine.slskmessages import newId
 from pynicotine.utils import CleanFile
-from pynicotine.utils import log
 from pynicotine.utils import unescape
 
 
@@ -114,10 +113,13 @@ class NetworkEventProcessor:
             shutil.move(config, corruptfile)
             short = _("Your config file is corrupt")
             long = _("We're sorry, but it seems your configuration file is corrupt. Please reconfigure Nicotine+.\n\nWe renamed your old configuration file to\n%(corrupt)s\nIf you open this file with a text editor you might be able to rescue some of your settings.") % {'corrupt': corruptfile}
-            log.addwarning(int)
             self.config = Config(config, data_dir)
             self.callback([PopupMessage(short, long)])
 
+        # These strings are accessed frequently. We store them to prevent requesting the translation every time.
+        self.conn_close_template = _("Connection closed by peer: %s")
+        self.conn_remove_template = _("Removed connection closed by peer: %(conn_obj)s %(address)s")
+
         self.bindip = bindip
         self.port = port
         self.config.frame = frame
@@ -129,6 +131,7 @@ class NetworkEventProcessor:
         self.ip_requested = []
         self.PrivateMessageQueue = {}
         self.users = {}
+        self.user_addr_requested = []
         self.queue = queue.Queue(0)
         self.shares = Shares(self)
 
@@ -168,7 +171,7 @@ class NetworkEventProcessor:
         self.servertimer = None
         self.servertimeout = -1
 
-        self.distribcache = {}
+        self.has_parent = False
         self.branchlevel = 0
         self.branchroot = None
 
@@ -258,7 +261,7 @@ class NetworkEventProcessor:
             slskmessages.FileSearch: self.SearchRequest,
             slskmessages.RoomSearch: self.RoomSearchRequest,
             slskmessages.UserSearch: self.SearchRequest,
-            slskmessages.NetInfo: self.NetInfo,
+            slskmessages.PossibleParents: self.PossibleParents,
             slskmessages.DistribAlive: self.DummyMessage,
             slskmessages.DistribSearch: self.DistribSearch,
             slskmessages.DistribServerSearch: self.DistribSearch,
@@ -308,10 +311,11 @@ class NetworkEventProcessor:
 
         conn = None
 
-        for i in self.peerconns:
-            if i.username == user and i.init.type == 'P' and message.__class__ is not slskmessages.FileRequest:
-                conn = i
-                break
+        if message.__class__ is not slskmessages.FileRequest:
+            for i in self.peerconns:
+                if i.username == user and i.init.type == 'P':
+                    conn = i
+                    break
 
         if conn is not None and conn.conn is not None:
 
@@ -323,7 +327,7 @@ class NetworkEventProcessor:
                 window.InitWindow(conn.username, conn.conn)
 
             if message.__class__ is slskmessages.TransferRequest and self.transfers is not None:
-                self.transfers.gotConnect(message.req, conn.conn)
+                self.transfers.gotConnect(message.req, conn.conn, message.direction)
 
             return
 
@@ -351,8 +355,9 @@ class NetworkEventProcessor:
 
             if firewalled:
                 if addr is None:
-                    self.queue.put(slskmessages.GetPeerAddress(user))
-                    addr = "in_progress"
+                    if user not in self.user_addr_requested:
+                        self.queue.put(slskmessages.GetPeerAddress(user))
+                        self.user_addr_requested.append(user)
                 elif behindfw is None:
                     self.queue.put(slskmessages.OutConn(None, addr))
                 else:
@@ -376,11 +381,11 @@ class NetworkEventProcessor:
         if message.__class__ is slskmessages.TransferRequest and self.transfers is not None:
 
             if conn.addr is None:
-                self.transfers.gettingAddress(message.req)
+                self.transfers.gettingAddress(message.req, message.direction)
             elif conn.token is None:
-                self.transfers.gotAddress(message.req)
+                self.transfers.gotAddress(message.req, message.direction)
             else:
-                self.transfers.gotConnectError(message.req)
+                self.transfers.gotConnectError(message.req, message.direction)
 
     def setServerTimer(self):
 
@@ -456,7 +461,7 @@ class NetworkEventProcessor:
 
         elif msg.connobj.__class__ is slskmessages.OutConn:
 
-            for i in self.peerconns[:]:
+            for i in self.peerconns:
 
                 if i.addr == msg.connobj.addr and i.conn is None:
 
@@ -470,7 +475,7 @@ class NetworkEventProcessor:
 
                         for j in i.msgs:
                             if j.__class__ is slskmessages.TransferRequest and self.transfers is not None:
-                                self.transfers.gotConnectError(j.req)
+                                self.transfers.gotConnectError(j.req, j.direction)
 
                         conntimeout = ConnectToPeerTimeout(i, self.callback)
                         timer = threading.Timer(120.0, conntimeout.timeout)
@@ -505,7 +510,7 @@ class NetworkEventProcessor:
         else:
             self.logMessage("%s %s %s" % (msg.err, msg.__class__, vars(msg)), 4)
 
-            self.ClosedConnection(msg.connobj.conn, msg.connobj.addr)
+            self.ClosedConnection(msg.connobj.conn, msg.connobj.addr, msg.err)
 
     def IncPort(self, msg):
         self.waitport = msg.port
@@ -520,7 +525,6 @@ class NetworkEventProcessor:
             }
         )
 
-        time.sleep(1)
         self.serverconn = msg.conn
         self.servertimeout = -1
         self.users = {}
@@ -559,7 +563,7 @@ class NetworkEventProcessor:
     def ConnClose(self, msg):
         self.ClosedConnection(msg.conn, msg.addr)
 
-    def ClosedConnection(self, conn, addr):
+    def ClosedConnection(self, conn, addr, error=None):
 
         if conn == self.serverconn:
 
@@ -591,30 +595,29 @@ class NetworkEventProcessor:
             self.frame.pluginhandler.ServerDisconnectNotification(userchoice)
 
         else:
-            for i in self.peerconns[:]:
+            for i in self.peerconns:
                 if i.conn == conn:
-                    self.logMessage(_("Connection closed by peer: %s") % vars(i), debugLevel=3)
+                    self.logMessage(self.conn_close_template % vars(i), debugLevel=3)
 
                     if i.conntimer is not None:
                         i.conntimer.cancel()
 
                     if self.transfers is not None:
-                        self.transfers.ConnClose(conn, addr)
+                        self.transfers.ConnClose(conn, addr, i.username, error)
 
-                    if i == self.GetDistribConn():
-                        self.DistribConnClosed(i)
+                    if i == self.GetParentConn():
+                        self.ParentConnClosed()
 
                     self.peerconns.remove(i)
                     break
             else:
                 self.logMessage(
-                    _("Removed connection closed by peer: %(conn_obj)s %(address)s") % {
+                    self.conn_remove_template % {
                         'conn_obj': conn,
                         'address': addr
                     },
                     3
                 )
-                self.queue.put(slskmessages.ConnClose(conn))
 
     def Login(self, msg):
 
@@ -623,7 +626,6 @@ class NetworkEventProcessor:
 
         if msg.success:
 
-            self.setStatus(_("Logged in, getting the list of rooms..."))
             self.transfers = transfers.Transfers(conf["transfers"]["downloads"], self.peerconns, self.queue, self, self.users)
 
             if msg.ip is not None:
@@ -640,8 +642,12 @@ class NetworkEventProcessor:
             for thing in self.config.sections["interests"]["dislikes"]:
                 self.queue.put(slskmessages.AddThingIHate(thing))
 
-            if not len(self.distribcache):
-                self.queue.put(slskmessages.HaveNoParent(1))
+            self.queue.put(slskmessages.HaveNoParent(1))
+
+            """ TODO: Nicotine+ can currently receive search requests from a parent connection, but
+            redirecting results to children is not implemented yet. Tell the server we don't accept
+            children for now. """
+            self.queue.put(slskmessages.AcceptChildren(0))
 
             self.queue.put(slskmessages.NotifyPrivileges(1, self.config.sections["server"]["login"]))
             self.privatechat.Login()
@@ -659,7 +665,7 @@ class NetworkEventProcessor:
     def ChangePassword(self, msg):
         password = msg.password
         self.config.sections["server"]["passw"] = password
-        self.config.writeConfig()
+        self.config.writeConfiguration()
         self.callback([PopupMessage(_("Your password has been changed"), "Password is %s" % password)])
 
     def NotifyPrivileges(self, msg):
@@ -750,15 +756,6 @@ class NetworkEventProcessor:
         if self.chatrooms is not None:
 
             self.chatrooms.roomsctrl.JoinRoom(msg)
-            ticker = ""
-
-            if msg.room in self.config.sections["ticker"]["rooms"]:
-                ticker = self.config.sections["ticker"]["rooms"][msg.room]
-            elif self.config.sections["ticker"]["default"]:
-                ticker = self.config.sections["ticker"]["default"]
-
-            if ticker:
-                self.queue.put(slskmessages.RoomTickerSet(msg.room, ticker))
 
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
@@ -902,8 +899,6 @@ class NetworkEventProcessor:
         if not msg.userexists:
             if msg.user not in self.users:
                 self.users[msg.user] = UserAddr(status=-1)
-            if self.search is not None:
-                self.search.NonExistantUser(msg.user)
 
         if self.transfers is not None:
             self.transfers.getAddUser(msg)
@@ -976,17 +971,13 @@ class NetworkEventProcessor:
         # TODO: Implement me
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
-    def DistribBranchLevel(self, msg):
-        # TODO: Implement me
-        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
-
     def DistribBranchRoot(self, msg):
         # TODO: Implement me
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
     def WishlistInterval(self, msg):
         if self.search is not None:
-            self.search.SetInterval(msg)
+            self.search.WishList.set_interval(msg)
         else:
             self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
@@ -1026,9 +1017,6 @@ class NetworkEventProcessor:
         if self.userbrowse is not None:
             self.userbrowse.GetUserStatus(msg)
 
-        if self.search is not None:
-            self.search.GetUserStatus(msg)
-
         if self.chatrooms is not None:
             self.chatrooms.roomsctrl.GetUserStatus(msg)
         else:
@@ -1076,8 +1064,10 @@ class NetworkEventProcessor:
 
     def GetPeerAddress(self, msg):
 
+        user = msg.user
+
         for i in self.peerconns:
-            if i.username == msg.user and i.addr == "in_progress":
+            if i.username == user and i.addr is None:
                 if msg.port != 0 or i.tryaddr == 10:
                     if i.tryaddr == 10:
                         self.logMessage(
@@ -1102,21 +1092,20 @@ class NetworkEventProcessor:
 
                     for j in i.msgs:
                         if j.__class__ is slskmessages.TransferRequest and self.transfers is not None:
-                            self.transfers.gotAddress(j.req)
+                            self.transfers.gotAddress(j.req, j.direction)
                 else:
                     if i.tryaddr is None:
                         i.tryaddr = 1
                         self.logMessage(
                             _("Server reported port 0 for user %(user)s, retrying") % {
-                                'user': msg.user
+                                'user': user
                             },
                             3
                         )
                     else:
                         i.tryaddr += 1
 
-                    self.queue.put(slskmessages.GetPeerAddress(msg.user))
-                break
+                    self.queue.put(slskmessages.GetPeerAddress(user))
         else:
 
             if msg.user in self.users:
@@ -1174,6 +1163,8 @@ class NetworkEventProcessor:
             self.logMessage(message)
             self.frame.pluginhandler.UserResolveNotification(msg.user, msg.ip, msg.port, cc)
 
+        self.user_addr_requested.remove(user)
+
     def Relogged(self, msg):
         self.logMessage(_("Someone else is logging in with the same nickname, server is going to disconnect us"))
         self.frame.manualdisconnect = 1
@@ -1205,7 +1196,7 @@ class NetworkEventProcessor:
                         self.transfers.gotFileConnect(j.req, msg.conn)
 
                     if j.__class__ is slskmessages.TransferRequest and self.transfers is not None:
-                        self.transfers.gotConnect(j.req, msg.conn)
+                        self.transfers.gotConnect(j.req, msg.conn, j.direction)
 
                     j.conn = msg.conn
                     self.queue.put(j)
@@ -1241,9 +1232,9 @@ class NetworkEventProcessor:
 
         if user in self.config.sections["server"]["banlist"]:
             if self.config.sections["transfers"]["usecustomban"]:
-                return 0, _("Banned (%s)") % self.config.sections["transfers"]["customban"]
+                return 0, "Banned (%s)" % self.config.sections["transfers"]["customban"]
             else:
-                return 0, _("Banned")
+                return 0, "Banned"
 
         if user in [i[0] for i in self.config.sections["server"]["userlist"]] and self.config.sections["transfers"]["enablebuddyshares"]:
             # For sending buddy-only shares
@@ -1253,10 +1244,10 @@ class NetworkEventProcessor:
             return 1, ""
 
         if self.config.sections["transfers"]["friendsonly"]:
-            return 0, _("Sorry, friends only")
+            return 0, "Sorry, friends only"
 
         if not self.config.sections["transfers"]["geoblock"]:
-            return 1, _("geoip")
+            return 1, ""
 
         cc = "-"
         if addr is not None:
@@ -1264,12 +1255,12 @@ class NetworkEventProcessor:
 
         if cc == "-":
             if self.config.sections["transfers"]["geopanic"]:
-                return 0, _("Sorry, geographical paranoia")
+                return 0, "Sorry, geographical paranoia"
             else:
                 return 1, ""
 
         if self.config.sections["transfers"]["geoblockcc"][0].find(cc) >= 0:
-            return 0, _("Sorry, your country is blocked")
+            return 0, "Sorry, your country is blocked"
 
         return 1, ""
 
@@ -1296,17 +1287,28 @@ class NetworkEventProcessor:
         return 0
 
     def ClosePeerConnection(self, peerconn):
+        try:
+            conn = peerconn.conn
+        except AttributeError:
+            conn = peerconn
 
-        if peerconn is None:
+        if conn is None:
             return
 
-        if not self.protothread.socketStillActive(peerconn):
-            self.queue.put(slskmessages.ConnClose(peerconn))
+        if not self.protothread.socketStillActive(conn):
+            self.queue.put(slskmessages.ConnClose(conn))
 
-            for i in self.peerconns:
-                if i.conn == peerconn:
-                    self.peerconns.remove(i)
-                    break
+            if type(peerconn) is socket:
+
+                for i in self.peerconns:
+                    if i.conn == peerconn:
+                        self.peerconns.remove(i)
+                        break
+            else:
+                try:
+                    self.peerconns.remove(peerconn)
+                except ValueError:
+                    pass
 
     def UserInfoReply(self, msg):
         for i in self.peerconns:
@@ -1379,12 +1381,7 @@ class NetworkEventProcessor:
             return
 
         try:
-            if sys.platform == "win32":
-                userpic = "%s" % self.config.sections["userinfo"]["pic"]
-                if not os.path.exists(userpic):
-                    userpic = self.config.sections["userinfo"]["pic"]
-            else:
-                userpic = self.config.sections["userinfo"]["pic"]
+            userpic = self.config.sections["userinfo"]["pic"]
 
             f = open(userpic, 'rb')
             pic = f.read()
@@ -1398,12 +1395,11 @@ class NetworkEventProcessor:
             totalupl = self.transfers.getTotalUploadsAllowed()
             queuesize = self.transfers.getUploadQueueSizes()[0]
             slotsavail = self.transfers.allowNewUploads()
-            ua = self.frame.np.config.sections["transfers"]["remotedownloads"]
 
-            if ua:
+            if self.frame.np.config.sections["transfers"]["remotedownloads"]:
                 uploadallowed = self.frame.np.config.sections["transfers"]["uploadallowed"]
             else:
-                uploadallowed = ua
+                uploadallowed = 0
 
             self.queue.put(slskmessages.UserInfoReply(msg.conn.conn, descr, pic, totalupl, queuesize, slotsavail, uploadallowed))
 
@@ -1432,7 +1428,7 @@ class NetworkEventProcessor:
                 country = ""
 
             self.search.ShowResult(msg, msg.user, country)
-            self.ClosePeerConnection(msg.conn.conn)
+            self.ClosePeerConnection(msg.conn)
 
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
@@ -1461,7 +1457,7 @@ class NetworkEventProcessor:
                         self.transfers.gotFileConnect(j.req, msg.conn.conn)
 
                     if j.__class__ is slskmessages.TransferRequest and self.transfers is not None:
-                        self.transfers.gotConnect(j.req, msg.conn.conn)
+                        self.transfers.gotConnect(j.req, msg.conn.conn, j.direction)
 
                     j.conn = msg.conn.conn
                     self.queue.put(j)
@@ -1473,15 +1469,15 @@ class NetworkEventProcessor:
 
     def CantConnectToPeer(self, msg):
 
-        for i in self.peerconns[:]:
+        for i in self.peerconns:
 
             if i.token == msg.token:
 
                 if i.conntimer is not None:
                     i.conntimer.cancel()
 
-                if i == self.GetDistribConn():
-                    self.DistribConnClosed(i)
+                if i == self.GetParentConn():
+                    self.ParentConnClosed()
 
                 self.peerconns.remove(i)
 
@@ -1493,22 +1489,21 @@ class NetworkEventProcessor:
                 break
 
     def ConnectToPeerTimeout(self, msg):
+        conn = msg.conn
 
-        for i in self.peerconns[:]:
+        if conn == self.GetParentConn():
+            self.ParentConnClosed()
 
-            if i == msg.conn:
+        try:
+            self.peerconns.remove(conn)
+        except ValueError:
+            pass
 
-                if i == self.GetDistribConn():
-                    self.DistribConnClosed(i)
+        self.logMessage(_("User %s does not respond to connect request, giving up") % (conn.username), 3)
 
-                self.peerconns.remove(i)
-
-                self.logMessage(_("User %s does not respond to connect request, giving up") % (i.username), 3)
-
-                for j in i.msgs:
-                    if j.__class__ in [slskmessages.TransferRequest, slskmessages.FileRequest] and self.transfers is not None:
-                        self.transfers.gotCantConnect(j.req)
-                break
+        for i in conn.msgs:
+            if i.__class__ in [slskmessages.TransferRequest, slskmessages.FileRequest] and self.transfers is not None:
+                self.transfers.gotCantConnect(i.req)
 
     def TransferTimeout(self, msg):
         if self.transfers is not None:
@@ -1588,29 +1583,30 @@ class NetworkEventProcessor:
 
     def FolderContentsResponse(self, msg):
         if self.transfers is not None:
+            conn = msg.conn.conn
+            file_list = msg.list
+
             # Check for a large number of files
             many = False
             folder = ""
-            files = []
 
-            for i in list(msg.list):
-                for j in list(msg.list[i]):
+            for i in file_list:
+                for j in file_list[i]:
                     if os.path.commonprefix([i, j]) == j:
-                        files = msg.list[i][j]
-                        numfiles = len(files)
+                        numfiles = len(file_list[i][j])
                         if numfiles > 100:
                             many = True
                             folder = j
 
             if many:
                 for i in self.peerconns:
-                    if i.conn is msg.conn.conn:
+                    if i.conn is conn:
                         username = i.username
                         break
 
-                self.frame.download_large_folder(username, folder, files, numfiles, msg)
+                self.frame.download_large_folder(username, folder, numfiles, conn, file_list)
             else:
-                self.transfers.FolderContentsResponse(msg)
+                self.transfers.FolderContentsResponse(conn, file_list)
         else:
             self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
@@ -1697,45 +1693,63 @@ class NetworkEventProcessor:
             self.shares.processSearchRequest(msg.searchterm, msg.user, msg.searchid, 0)
         self.frame.pluginhandler.DistribSearchNotification(msg.searchterm, msg.user, msg.searchid)
 
-    def NetInfo(self, msg):
+    def PossibleParents(self, msg):
 
-        self.distribcache.update(msg.list)
+        """ Server sent a list of 10 potential parents, whose purpose is to forward us search requests.
+        We attempt to connect to them all at once, since connection errors are fairly common. """
 
-        if len(self.distribcache) > 0:
+        potential_parents = msg.list
 
-            self.queue.put(slskmessages.HaveNoParent(0))
+        if potential_parents:
 
-            if not self.GetDistribConn():
-
-                user = list(self.distribcache.keys())[0]
-                addr = self.distribcache[user]
-
-                self.queue.put(slskmessages.SearchParent(addr[0]))
+            for user in potential_parents:
+                addr = potential_parents[user]
 
                 self.ProcessRequestToPeer(user, slskmessages.DistribConn(), None, addr)
 
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
-    def GetDistribConn(self):
+    def GetParentConn(self):
         for i in self.peerconns:
             if i.init.type == 'D':
                 return i
+
         return None
 
-    def DistribConnClosed(self, conn):
+    def ParentConnClosed(self):
+        """ Tell the server it needs to send us a NetInfo message with a new list of
+        potential parents. """
 
-        del self.distribcache[conn.username]
+        self.has_parent = False
+        self.queue.put(slskmessages.HaveNoParent(1))
 
-        if len(self.distribcache) > 0:
+    def DistribBranchLevel(self, msg):
+        """ This message is received when we have a successful connection with a potential
+        parent. Tell the server who our parent is, and stop requesting new potential parents. """
 
-            user = list(self.distribcache.keys())[0]
-            addr = self.distribcache[user]
+        if not self.has_parent:
 
-            self.queue.put(slskmessages.SearchParent(addr[0]))
+            for i in self.peerconns[:]:
+                if i.init.type == 'D':
+                    """ We previously attempted to connect to all potential parents. Since we now
+                    have a parent, stop connecting to the others. """
 
-            self.ProcessRequestToPeer(user, slskmessages.DistribConn(), None, addr)
-        else:
-            self.queue.put(slskmessages.HaveNoParent(1))
+                    if i.conn != msg.conn.conn:
+                        if i.conn is not None:
+                            self.queue.put(slskmessages.ConnClose(i.conn))
+
+                        self.peerconns.remove(i)
+
+            parent = self.GetParentConn()
+
+            if parent is not None:
+                self.queue.put(slskmessages.HaveNoParent(0))
+                self.queue.put(slskmessages.SearchParent(msg.conn.addr[0]))
+                self.has_parent = True
+            else:
+                self.ParentConnClosed()
+
+        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
     def GlobalRecommendations(self, msg):
         self.frame.GlobalRecommendations(msg)
