@@ -34,7 +34,6 @@ import os.path
 import re
 import shutil
 import stat
-import sys
 import threading
 import time
 from gettext import gettext as _
@@ -49,8 +48,6 @@ from pynicotine.utils import executeCommand
 from pynicotine.utils import CleanFile
 from pynicotine.utils import GetResultBitrateLength
 
-win32 = sys.platform.startswith("win")
-
 
 class Transfer(object):
     """ This class holds information about a single transfer. """
@@ -126,7 +123,7 @@ class Transfers:
         self.eventprocessor = eventprocessor
         self.downloads = []
         self.uploads = []
-        self.privilegedusers = []
+        self.privilegedusers = set()
         self.RequestedUploadQueue = []
         getstatus = {}
 
@@ -171,10 +168,9 @@ class Transfers:
             )
             getstatus[i[0]] = ""
 
-        for i in list(getstatus.keys()):
+        for i in getstatus:
             if i not in self.eventprocessor.watchedusers:
                 self.queue.put(slskmessages.AddUser(i))
-            self.queue.put(slskmessages.GetUserStatus(i))
 
         self.users = users
         self.downloadspanel = None
@@ -199,8 +195,7 @@ class Transfers:
 
     def addToPrivileged(self, user):
 
-        if user not in self.privilegedusers:
-            self.privilegedusers.append(user)
+        self.privilegedusers.add(user)
 
         if user in self.usersqueued:
             self.privusersqueued.setdefault(user, 0)
@@ -335,46 +330,61 @@ class Transfers:
                 )
                 break
 
-    def gettingAddress(self, req):
+    def gettingAddress(self, req, direction):
 
-        for i in self.downloads:
-            if i.req == req:
-                i.status = "Getting address"
-                self.downloadspanel.update(i)
+        if direction == 0:
+            for i in self.downloads:
+                if i.req == req:
+                    i.status = "Getting address"
+                    self.downloadspanel.update(i)
+                    break
 
-        for i in self.uploads:
-            if i.req == req:
-                i.status = "Getting address"
-                self.uploadspanel.update(i)
+        elif direction == 1:
 
-    def gotAddress(self, req):
+            for i in self.uploads:
+                if i.req == req:
+                    i.status = "Getting address"
+                    self.uploadspanel.update(i)
+                    break
+
+    def gotAddress(self, req, direction):
         """ A connection is in progress, we got the address for a user we need
         to connect to."""
 
-        for i in self.downloads:
-            if i.req == req:
-                i.status = "Connecting"
-                self.downloadspanel.update(i)
+        if direction == 0:
+            for i in self.downloads:
+                if i.req == req:
+                    i.status = "Connecting"
+                    self.downloadspanel.update(i)
+                    break
 
-        for i in self.uploads:
-            if i.req == req:
-                i.status = "Connecting"
-                self.uploadspanel.update(i)
+        elif direction == 1:
 
-    def gotConnectError(self, req):
+            for i in self.uploads:
+                if i.req == req:
+                    i.status = "Connecting"
+                    self.uploadspanel.update(i)
+                    break
+
+    def gotConnectError(self, req, direction):
         """ We couldn't connect to the user, now we are waitng for him to
         connect to us. Note that all this logic is handled by the network
         event processor, we just provide a visual feedback to the user."""
 
-        for i in self.downloads:
-            if i.req == req:
-                i.status = "Waiting for peer to connect"
-                self.downloadspanel.update(i)
+        if direction == 0:
+            for i in self.downloads:
+                if i.req == req:
+                    i.status = "Waiting for peer to connect"
+                    self.downloadspanel.update(i)
+                    break
 
-        for i in self.uploads:
-            if i.req == req:
-                i.status = "Waiting for peer to connect"
-                self.uploadspanel.update(i)
+        elif direction == 1:
+
+            for i in self.uploads:
+                if i.req == req:
+                    i.status = "Waiting for peer to connect"
+                    self.uploadspanel.update(i)
+                    break
 
     def gotCantConnect(self, req):
         """ We can't connect to the user, either way. """
@@ -382,10 +392,12 @@ class Transfers:
         for i in self.downloads:
             if i.req == req:
                 self._getCantConnectDownload(i)
+                break
 
         for i in self.uploads:
             if i.req == req:
                 self._getCantConnectUpload(i)
+                break
 
     def _getCantConnectDownload(self, i):
 
@@ -424,32 +436,38 @@ class Transfers:
             if i.req == req:
                 i.status = "Initializing transfer"
                 self.downloadspanel.update(i)
+                break
 
         for i in self.uploads:
             if i.req == req:
                 i.status = "Initializing transfer"
                 self.uploadspanel.update(i)
+                break
 
-    def gotConnect(self, req, conn):
+    def gotConnect(self, req, conn, direction):
         """ A connection has been established, now exchange initialisation
         messages."""
 
-        for i in self.downloads:
-            if i.req == req:
-                i.status = "Requesting file"
-                i.requestconn = conn
-                self.downloadspanel.update(i)
+        if direction == 0:
+            for i in self.downloads:
+                if i.req == req:
+                    i.status = "Requesting file"
+                    i.requestconn = conn
+                    self.downloadspanel.update(i)
+                    break
 
-        for i in self.uploads:
-            if i.req == req:
-                i.status = "Requesting file"
-                i.requestconn = conn
-                self.uploadspanel.update(i)
+        elif direction == 1:
+
+            for i in self.uploads:
+                if i.req == req:
+                    i.status = "Requesting file"
+                    i.requestconn = conn
+                    self.uploadspanel.update(i)
+                    break
 
     def TransferRequest(self, msg):
 
         user = response = None
-        transfers = self.eventprocessor.config.sections["transfers"]  # noqa: F841
 
         if msg.conn is not None:
             for i in self.peerconns:
@@ -479,7 +497,7 @@ class Transfers:
     def TransferRequestDownloads(self, msg, user, conn, addr):
 
         for i in self.downloads:
-            if i.filename == msg.file and user == i.user and i.status == "Queued":
+            if i.filename == msg.file and user == i.user and i.status not in ["Aborted", "Paused"]:
                 # Remote peer is signalling a tranfer is ready, attempting to download it
 
                 """ If the file is larger than 2GB, the SoulseekQt client seems to
@@ -491,7 +509,7 @@ class Transfers:
 
                 i.req = msg.req
                 i.status = "Waiting for download"
-                transfertimeout = TransferTimeout(i.req, self.eventprocessor.frame.callback)
+                transfertimeout = TransferTimeout(i.req, self.eventprocessor.frame.networkcallback)
 
                 if i.transfertimer is not None:
                     i.transfertimer.cancel()
@@ -521,8 +539,7 @@ class Transfers:
                     self.queue.put(slskmessages.AddUser(user))
 
                 self.queue.put(slskmessages.GetUserStatus(user))
-                if user != self.eventprocessor.config.sections["server"]["login"]:
-                    response = slskmessages.TransferResponse(conn, 0, reason="Queued", req=transfer.req)
+                response = slskmessages.TransferResponse(conn, 0, reason="Queued", req=transfer.req)
                 self.downloadspanel.update(transfer)
             else:
                 response = slskmessages.TransferResponse(conn, 0, reason="Cancelled", req=msg.req)
@@ -562,7 +579,7 @@ class Transfers:
             return slskmessages.TransferResponse(conn, 0, reason="Queued", req=msg.req)
 
         # Has user hit queue limit?
-        friend = user in [i[0] for i in self.eventprocessor.userlist.userlist]
+        friend = user in [i[0] for i in self.eventprocessor.config.sections["server"]["userlist"]]
         if friend and self.eventprocessor.config.sections["transfers"]["friendsnolimits"]:
             limits = False
         else:
@@ -599,7 +616,7 @@ class Transfers:
         size = self.getFileSize(realpath)
         response = slskmessages.TransferResponse(conn, 1, req=msg.req, filesize=size)
 
-        transfertimeout = TransferTimeout(msg.req, self.eventprocessor.frame.callback)
+        transfertimeout = TransferTimeout(msg.req, self.eventprocessor.frame.networkcallback)
         transferobj = Transfer(
             user=user, realfilename=realpath, filename=msg.file,
             path=os.path.dirname(realpath), status="Waiting for upload",
@@ -637,9 +654,8 @@ class Transfers:
         if not uploadslimit:
             return False
 
-        sizelist = [i.size for i in self.uploads if i.user == user and i.status == "Queued"]
+        size = sum(i.size for i in self.uploads if i.user == user and i.status == "Queued")
 
-        size = sum(sizelist)
         return size >= uploadslimit
 
     def fileLimitReached(self, user):
@@ -649,7 +665,7 @@ class Transfers:
         if not filelimit:
             return False
 
-        numfiles = len([i for i in self.uploads if i.user == user and i.status == "Queued"])
+        numfiles = sum(1 for i in self.uploads if i.user == user and i.status == "Queued")
 
         return numfiles >= filelimit
 
@@ -669,7 +685,7 @@ class Transfers:
 
         if not self.fileIsUploadQueued(user, msg.file):
 
-            friend = user in [i[0] for i in self.eventprocessor.userlist.userlist]
+            friend = user in [i[0] for i in self.eventprocessor.config.sections["server"]["userlist"]]
             if friend and self.eventprocessor.config.sections["transfers"]["friendsnolimits"]:
                 limits = 0
             else:
@@ -751,7 +767,7 @@ class Transfers:
             # Remote Uploads only for users in list
             if transfers["uploadallowed"] == 2:
                 # Users in userlist
-                if user not in [i[0] for i in self.eventprocessor.userlist.userlist]:
+                if user not in [i[0] for i in self.eventprocessor.config.sections["server"]["userlist"]]:
                     # Not a buddy
                     return False
 
@@ -765,10 +781,12 @@ class Transfers:
 
             if transfers["uploadallowed"] == 3:
                 # Trusted Users
-                if user not in [i[0] for i in self.eventprocessor.userlist.userlist]:
+                userlist = [i[0] for i in self.eventprocessor.config.sections["server"]["userlist"]]
+
+                if user not in userlist:
                     # Not a buddy
                     return False
-                if user not in self.eventprocessor.userlist.trusted:
+                if not self.eventprocessor.config.sections["server"]["userlist"][userlist.index(user)][4]:
                     # Not Trusted
                     return False
 
@@ -781,27 +799,25 @@ class Transfers:
         for i in self.peerconns:
             if i.conn is msg.conn.conn:
                 user = i.username
+                break
 
         for i in self.downloads:
-            if i.user == user and i.filename == msg.file and i.status == "Queued":
+            if i.user == user and i.filename == msg.file and i.status not in ["Aborted", "Paused"]:
+                if i.status in self.TRANSFER:
+                    self.AbortTransfer(i, reason=msg.reason)
+
                 i.status = msg.reason
                 self.downloadspanel.update(i)
+
                 break
 
     def fileIsShared(self, user, virtualfilename, realfilename):
 
-        if win32:
-            u_realfilename = "%s" % realfilename
-            u_virtualfilename = "%s" % virtualfilename
-        else:
-            u_realfilename = realfilename
-            u_virtualfilename = virtualfilename
-
-        u_realfilename = u_realfilename.replace("\\", os.sep)
-        if not os.access(u_realfilename, os.R_OK):
+        realfilename = realfilename.replace("\\", os.sep)
+        if not os.access(realfilename, os.R_OK):
             return False
 
-        (dir, sep, file) = u_virtualfilename.rpartition('\\')
+        (dir, sep, file) = virtualfilename.rpartition('\\')
 
         if self.eventprocessor.config.sections["transfers"]["enablebuddyshares"]:
             if user in [i[0] for i in self.eventprocessor.config.sections["server"]["userlist"]]:
@@ -845,24 +861,28 @@ class Transfers:
         limit_upload_slots = self.eventprocessor.config.sections["transfers"]["useupslots"]
         limit_upload_speed = self.eventprocessor.config.sections["transfers"]["uselimit"]
 
-        bandwidthlist = [i.speed for i in self.uploads if i.conn is not None and i.speed is not None]
+        bandwidth_sum = sum(i.speed for i in self.uploads if i.conn is not None and i.speed is not None)
         currently_negotiating = self.transferNegotiating()
 
         if limit_upload_slots:
             maxupslots = self.eventprocessor.config.sections["transfers"]["uploadslots"]
-            if len(bandwidthlist) + currently_negotiating >= maxupslots:
+            in_progress_count = sum(1 for i in self.uploads if i.conn is not None and i.speed is not None)
+
+            if in_progress_count + currently_negotiating >= maxupslots:
                 return False
 
         if limit_upload_speed:
             max_upload_speed = self.eventprocessor.config.sections["transfers"]["uploadlimit"]
-            if sum(bandwidthlist) >= max_upload_speed:
+
+            if bandwidth_sum >= max_upload_speed:
                 return False
+
             if currently_negotiating:
                 return False
 
         maxbandwidth = self.eventprocessor.config.sections["transfers"]["uploadbandwidth"]
         if maxbandwidth:
-            if sum(bandwidthlist) >= maxbandwidth:
+            if bandwidth_sum >= maxbandwidth:
                 return False
 
         return True
@@ -870,10 +890,7 @@ class Transfers:
     def getFileSize(self, filename):
 
         try:
-            if win32:
-                size = os.path.getsize("%s" % filename.replace("\\", os.sep))
-            else:
-                size = os.path.getsize(filename.replace("\\", os.sep))
+            size = os.path.getsize(filename.replace("\\", os.sep))
         except Exception:
             # file doesn't exist (remote files are always this)
             size = 0
@@ -904,6 +921,7 @@ class Transfers:
                     self.eventprocessor.ProcessRequestToPeer(i.user, slskmessages.PlaceInQueueRequest(None, i.filename))
 
                 self.checkUploadQueue()
+                break
 
             for i in self.uploads:
 
@@ -932,6 +950,7 @@ class Transfers:
                     self.AutoClearUpload(i)
 
                 self.checkUploadQueue()
+                break
 
         elif msg.filesize is not None:
             for i in self.downloads:
@@ -987,6 +1006,8 @@ class Transfers:
             elif i in self.uploads:
                 self.uploadspanel.update(i)
 
+            break
+
         self.checkUploadQueue()
 
     def FileRequest(self, msg):
@@ -1003,8 +1024,6 @@ class Transfers:
                 self._FileRequestUpload(msg, i)
                 return
 
-        self.eventprocessor.logMessage(_("Unknown file request: %s") % str(vars(msg)), 1)
-
         self.queue.put(slskmessages.ConnClose(msg.conn))
 
     def _FileRequestDownload(self, msg, i):
@@ -1037,7 +1056,7 @@ class Transfers:
                 i.status = "Download directory error"
                 i.conn = None
                 self.queue.put(slskmessages.ConnClose(msg.conn))
-                self.eventprocessor.frame.NewNotification(_("OS error: %s") % strerror, title=_("Folder download error"))
+                self.eventprocessor.frame.Notifications.NewNotificationPopup(_("OS error: %s") % strerror, title=_("Folder download error"))
 
             else:
                 # also check for a windows-style incomplete transfer
@@ -1057,10 +1076,7 @@ class Transfers:
                     else:
                         fname = pynewfname
 
-                    if win32:
-                        f = open("%s" % fname, 'ab+')
-                    else:
-                        f = open(fname, 'ab+')
+                    f = open(fname, 'ab+')
 
                 except IOError as strerror:
                     self.eventprocessor.logMessage(_("Download I/O error: %s") % strerror)
@@ -1097,7 +1113,7 @@ class Transfers:
                         self.queue.put(slskmessages.DownloadFile(i.conn, size, f, i.size))
                         self.eventprocessor.logMessage(_("Download started: %s") % ("%s" % f.name), 5)
 
-                        self.eventprocessor.logTransfer(_("Download started: user %(user)s, file %(file)s") % {'user': i.user, 'file': "%s" % f.name}, 5)
+                        self.eventprocessor.logTransfer(_("Download started: user %(user)s, file %(file)s") % {'user': i.user, 'file': "%s" % f.name}, 1)
                     else:
                         self.DownloadFinished(f, i)
                         needupdate = False
@@ -1126,10 +1142,7 @@ class Transfers:
 
             try:
                 # Open File
-                if win32:
-                    filename = "%s" % i.realfilename.replace("\\", os.sep)
-                else:
-                    filename = i.realfilename.replace("\\", os.sep)
+                filename = i.realfilename.replace("\\", os.sep)
 
                 f = open(filename, "rb")
                 self.queue.put(slskmessages.UploadFile(i.conn, file=f, size=i.size))
@@ -1216,7 +1229,9 @@ class Transfers:
                 oldelapsed = i.timeelapsed
                 i.timeelapsed = curtime - i.starttime
 
-                if curtime > i.starttime and i.currentbytes > i.offset:
+                if curtime > i.starttime and \
+                        i.currentbytes > i.lastbytes:
+
                     try:
                         i.speed = max(0, (i.currentbytes - i.lastbytes) / (curtime - i.lasttime) / 1024)
                     except ZeroDivisionError:
@@ -1232,7 +1247,6 @@ class Transfers:
                 if i.size > i.currentbytes:
                     if oldelapsed == i.timeelapsed:
                         needupdate = False
-                    i.status = "Transferring"
                 else:
                     self.DownloadFinished(msg.file, i)
                     needupdate = False
@@ -1249,6 +1263,8 @@ class Transfers:
             if needupdate:
                 self.downloadspanel.update(i)
 
+            break
+
     def DownloadFinished(self, file, i):
         file.close()
         i.file = None
@@ -1265,75 +1281,53 @@ class Transfers:
         if not os.access(folder, os.F_OK):
             os.makedirs(folder)
 
-        (newname, identicalfile) = self.getRenamed(os.path.join(folder, basename), file.name)
+        newname = self.getRenamed(os.path.join(folder, basename))
 
-        if newname:
-            try:
-                shutil.move(file.name, newname)
-            except (IOError, OSError) as inst:  # noqa: F841
-                try:
-                    shutil.move(file.name, "%s" % newname)
-                except (IOError, OSError) as inst:
-                    log.addwarning(
-                        _("Couldn't move '%(tempfile)s' to '%(file)s': %(error)s") % {
-                            'tempfile': file.name,
-                            'file': newname,
-                            'error': inst
-                        }
-                    )
+        try:
+            shutil.move(file.name, newname)
+        except (IOError, OSError) as inst:
+            log.addwarning(
+                _("Couldn't move '%(tempfile)s' to '%(file)s': %(error)s") % {
+                    'tempfile': "%s" % file.name,
+                    'file': newname,
+                    'error': inst
+                }
+            )
 
         i.status = "Finished"
         i.speed = 0
         i.timeleft = ""
 
-        if newname:
-            self.eventprocessor.logMessage(
-                _("Download finished: %(file)s") % {
-                    'file': newname
-                },
-                5
-            )
-            self.eventprocessor.logTransfer(
-                _("Download finished: user %(user)s, file %(file)s") % {
-                    'user': i.user,
-                    'file': i.filename
-                },
-                1
-            )
-        else:
-            self.eventprocessor.logMessage(
-                _("File %(file)s is identical to %(identical)s, not saving.") % {
-                    'file': file.name,
-                    'identical': identicalfile
-                },
-                1
-            )
-            self.eventprocessor.logTransfer(
-                _("Download finished but not saved since it's a duplicate: user %(user)s, file %(file)s") % {
-                    'user': i.user,
-                    'file': i.filename
-                },
-                1
-            )
+        self.eventprocessor.logMessage(
+            _("Download finished: %(file)s") % {
+                'file': newname
+            },
+            5
+        )
+
+        self.eventprocessor.logTransfer(
+            _("Download finished: user %(user)s, file %(file)s") % {
+                'user': i.user,
+                'file': i.filename
+            },
+            1
+        )
 
         self.queue.put(slskmessages.ConnClose(i.conn))
         i.conn = None
 
-        if newname:
-            if win32:
-                self.addToShared("%s" % newname)
-            else:
-                self.addToShared(newname)
-            self.eventprocessor.shares.sendNumSharedFoldersFiles()
+        self.addToShared(newname)
+        self.eventprocessor.shares.sendNumSharedFoldersFiles()
 
-            if config["transfers"]["shownotification"]:
-                self.eventprocessor.frame.NewNotification(
-                    _("%(file)s downloaded from %(user)s") % {
-                        'user': i.user,
-                        'file': newname.rsplit(os.sep, 1)[1]
-                    },
-                    title=_("File downloaded")
-                )
+        if config["notifications"]["notification_popup_file"]:
+            self.eventprocessor.frame.Notifications.NewNotificationPopup(
+                _("%(file)s downloaded from %(user)s") % {
+                    'user': i.user,
+                    'file': newname.rsplit(os.sep, 1)[1]
+                },
+                title=_("File downloaded"),
+                soundnamenotify="complete-download"
+            )
 
         self.SaveDownloads()
 
@@ -1341,26 +1335,27 @@ class Transfers:
         if not self.AutoClearDownload(i):
             self.downloadspanel.update(i)
 
-        if newname and config["transfers"]["afterfinish"]:
+        if config["transfers"]["afterfinish"]:
             if not executeCommand(config["transfers"]["afterfinish"], newname):
                 self.eventprocessor.logMessage(_("Trouble executing '%s'") % config["transfers"]["afterfinish"])
             else:
                 self.eventprocessor.logMessage(_("Executed: %s") % config["transfers"]["afterfinish"])
 
-        if i.path and (config["transfers"]["shownotificationperfolder"] or config["transfers"]["afterfolder"]):
+        if i.path and (config["notifications"]["notification_popup_folder"] or config["transfers"]["afterfolder"]):
 
             # walk through downloads and break if any file in the same folder exists, else execute
             for ia in self.downloads:
                 if ia.status not in ["Finished", "Aborted", "Paused", "Filtered"] and ia.path and ia.path == i.path:
                     break
             else:
-                if config["transfers"]["shownotificationperfolder"]:
-                    self.eventprocessor.frame.NewNotification(
+                if config["notifications"]["notification_popup_folder"]:
+                    self.eventprocessor.frame.Notifications.NewNotificationPopup(
                         _("%(folder)s downloaded from %(user)s") % {
                             'user': i.user,
                             'folder': folder
                         },
-                        title=_("Folder downloaded")
+                        title=_("Folder downloaded"),
+                        soundnamenotify="complete-download"
                     )
                 if config["transfers"]["afterfolder"]:
                     if not executeCommand(config["transfers"]["afterfolder"], folder):
@@ -1399,7 +1394,9 @@ class Transfers:
             oldelapsed = i.timeelapsed
             i.timeelapsed = curtime - i.starttime
 
-            if curtime > i.starttime and i.currentbytes > i.offset:
+            if curtime > i.starttime and \
+                    i.currentbytes > i.lastbytes:
+
                 try:
                     i.speed = max(0, (i.currentbytes - i.lastbytes) / (curtime - i.lasttime) / 1024)
                 except ZeroDivisionError:
@@ -1431,14 +1428,16 @@ class Transfers:
                 self.checkUploadQueue()
                 sleep(0.01)
             else:
+                if i.speed is not None:
+                    speedbytes = int(i.speed * 1024)
+                    self.eventprocessor.speed = speedbytes
+                    self.queue.put(slskmessages.SendUploadSpeed(speedbytes))
+
                 msg.file.close()
                 i.status = "Finished"
                 i.speed = 0
                 i.timeleft = ""
 
-                if i.speed is not None:
-                    self.queue.put(slskmessages.SendUploadSpeed(int(i.speed * 1024)))
-
                 for j in self.uploads:
                     if j.user == i.user:
                         j.timequeued = curtime
@@ -1460,6 +1459,8 @@ class Transfers:
             if needupdate:
                 self.uploadspanel.update(i)
 
+            break
+
     def AutoClearDownload(self, transfer):
         if self.eventprocessor.config.sections["transfers"]["autoclear_downloads"]:
             self.downloads.remove(transfer)
@@ -1489,23 +1490,27 @@ class Transfers:
         else:
             banmsg = _("Banned")
 
-        list = [i for i in self.uploads if i.user == user]
-        for upload in list:
+        for upload in self.uploads:
+            if upload.user != user:
+                continue
+
             if upload.status == "Queued":
                 self.eventprocessor.ProcessRequestToPeer(user, slskmessages.QueueFailed(None, file=upload.filename, reason=banmsg))
             else:
-                self.AbortTransfer(upload)
+                self.AbortTransfer(upload, reason=banmsg)
 
         if self.uploadspanel is not None:
             self.uploadspanel.ClearByUser(user)
         if user not in self.eventprocessor.config.sections["server"]["banlist"]:
             self.eventprocessor.config.sections["server"]["banlist"].append(user)
-            self.eventprocessor.config.writeConfig()
+            self.eventprocessor.config.writeConfiguration()
+            self.eventprocessor.config.writeDownloadQueue()
 
     def startCheckDownloadQueueTimer(self):
         GLib.timeout_add(60000, self.checkDownloadQueue)
 
-    # Find failed or stuck downloads and attempt to queue them
+    # Find failed or stuck downloads and attempt to queue them.
+    # Also ask for the queue position of downloads.
     def checkDownloadQueue(self):
 
         statuslist = self.FAILED_TRANSFERS + \
@@ -1515,6 +1520,8 @@ class Transfers:
             if transfer.status in statuslist:
                 self.AbortTransfer(transfer)
                 self.getFile(transfer.user, transfer.filename, transfer.path, transfer)
+            elif transfer.status == "Queued":
+                self.eventprocessor.ProcessRequestToPeer(transfer.user, slskmessages.PlaceInQueueRequest(None, transfer.filename))
 
         self.startCheckDownloadQueueTimer()
 
@@ -1683,7 +1690,7 @@ class Transfers:
                     count += 1
             return count, count
         else:
-            if self.isPrivileged(username):
+            if username is not None and self.isPrivileged(username):
                 return len(self.privusersqueued), len(self.privusersqueued)
             else:
                 return len(self.usersqueued) + self.privcount, self.privcount
@@ -1718,17 +1725,17 @@ class Transfers:
             maxupslots = self.eventprocessor.config.sections["transfers"]["uploadslots"]
             return maxupslots
         else:
-            lst = [i for i in self.uploads if i.conn is not None]
+            lstlen = sum(1 for i in self.uploads if i.conn is not None)
             if self.allowNewUploads():
-                return len(lst) + 1
+                return lstlen + 1
             else:
-                return len(lst)
+                return lstlen
 
     def UserListPrivileged(self, user):
 
         # All users
         if self.eventprocessor.config.sections["transfers"]["preferfriends"]:
-            return user in [i[0] for i in self.eventprocessor.config.sections["server"]["userlist"]]
+            return any(user in i[0] for i in self.eventprocessor.config.sections["server"]["userlist"])
 
         # Only privileged users
         userlist = [i[0] for i in self.eventprocessor.config.sections["server"]["userlist"]]
@@ -1747,7 +1754,7 @@ class Transfers:
         else:
             return False
 
-    def ConnClose(self, conn, addr):
+    def ConnClose(self, conn, addr, user, error):
         """ The remote user has closed the connection either because
         he logged off, or because there's a network problem. """
 
@@ -1758,7 +1765,10 @@ class Transfers:
             self._ConnClose(conn, addr, i, "download")
 
         for i in self.uploads:
-            if i.conn != conn:
+            if type(error) is not ConnectionRefusedError and i.conn != conn:
+                continue
+            elif i.user != user:
+                # Connection refused, cancel all of user's transfers
                 continue
 
             self._ConnClose(conn, addr, i, "upload")
@@ -1803,47 +1813,21 @@ class Transfers:
 
         self.checkUploadQueue()
 
-    def getRenamed(self, name, originalfile):
+    def getRenamed(self, name):
         """ When a transfer is finished, we remove INCOMPLETE~ or INCOMPLETE
         prefix from the file's name.
 
-        Returns a tuple (newname, identicalfile) where precisely one of the two
-        is None, and the other is a complete path. If newname is None a file
-        with the same checksum value already exists as identicalfile, if
-        identicalfile is None the file can be saved under newname."""
+        Checks if a file with the same name already exists, and adds a number
+        to the file name if that's the case. """
 
-        if win32 and not os.path.exists("%s" % name) and not os.path.exists(name):
-            # Filename doesn't exist, good for renaming
-            return (name, None)
-        elif not win32 and not os.path.exists(name):
-            return (name, None)
+        filename, extension = os.path.splitext(name)
+        counter = 1
 
-        # A file with the same name already exists. First lets check whether it's a duplicate
-        ourchecksum = self.getChecksum(originalfile)
-        newname = name
+        while os.path.exists(name):
+            name = filename + " (" + str(counter) + ")" + extension
+            counter += 1
 
-        n = 1
-        while n < 1000:
-            existingchecksum = self.getChecksum(newname)
-            if ourchecksum == existingchecksum:
-                return (None, newname)
-            newname = name + "." + str(n)
-            if not os.path.exists(newname):
-                break
-            n += 1
-
-        return (newname, None)
-
-    def getChecksum(self, path):
-        try:
-            h = open(path, 'rb')
-            m = hashlib.md5()
-            m.update(h.read(-1))
-            digest = m.digest()
-            h.close()
-            return digest
-        except IOError:
-            return None
+        return name
 
     def PlaceInQueue(self, msg):
         """ The server tells us our place in queue for a particular transfer."""
@@ -1864,6 +1848,7 @@ class Transfers:
 
                 i.place = msg.place
                 self.downloadspanel.update(i)
+                break
 
     def FileError(self, msg):
         """ Networking thread encountered a local file error"""
@@ -1890,35 +1875,35 @@ class Transfers:
 
             self.checkUploadQueue()
 
-    def FolderContentsResponse(self, msg):
+    def FolderContentsResponse(self, conn, file_list):
         """ When we got a contents of a folder, get all the files in it, but
         skip the files in subfolders"""
 
         username = None
         for i in self.peerconns:
-            if i.conn is msg.conn.conn:
+            if i.conn is conn:
                 username = i.username
                 break
 
         if username is None:
             return
 
-        for i in list(msg.list):
-            for directory in list(msg.list[i]):
+        for i in file_list:
+            for directory in file_list[i]:
 
                 if os.path.commonprefix([i, directory]) == directory:
                     priorityfiles = []
                     normalfiles = []
 
                     if self.eventprocessor.config.sections["transfers"]["prioritize"]:
-                        for file in msg.list[i][directory]:
+                        for file in file_list[i][directory]:
                             parts = file[1].rsplit('.', 1)
                             if len(parts) == 2 and parts[1] in ['sfv', 'md5', 'nfo']:
                                 priorityfiles.append(file)
                             else:
                                 normalfiles.append(file)
                     else:
-                        normalfiles = msg.list[i][directory][:]
+                        normalfiles = file_list[i][directory][:]
 
                     if self.eventprocessor.config.sections["transfers"]["reverseorder"]:
                         deco = [(x[1], x) for x in normalfiles]
@@ -1964,6 +1949,23 @@ class Transfers:
             parent = directory.split('\\')[-1]
 
         destination = os.path.join(destination, parent)
+
+        if destination[0] != '/':
+            destination = os.path.join(
+                self.eventprocessor.config.sections["transfers"]["downloaddir"],
+                destination
+            )
+
+        """ Make sure the target folder doesn't exist
+        If it exists, append a number to the folder name """
+
+        orig_destination = destination
+        counter = 1
+
+        while os.path.exists(destination):
+            destination = orig_destination + " (" + str(counter) + ")"
+            counter += 1
+
         return destination
 
     def AbortTransfers(self):
@@ -1977,14 +1979,14 @@ class Transfers:
                 self.AbortTransfer(i)
                 i.status = "Old"
 
-    def AbortTransfer(self, transfer, remove=0):
+    def AbortTransfer(self, transfer, remove=0, reason="Aborted"):
 
         transfer.req = None
         transfer.speed = 0
         transfer.timeleft = ""
 
         if transfer in self.uploads:
-            self.eventprocessor.ProcessRequestToPeer(transfer.user, slskmessages.QueueFailed(None, file=transfer.filename, reason="Aborted"))
+            self.eventprocessor.ProcessRequestToPeer(transfer.user, slskmessages.QueueFailed(None, file=transfer.filename, reason=reason))
 
         if transfer.conn is not None:
             self.queue.put(slskmessages.ConnClose(transfer.conn))
