@@ -29,13 +29,13 @@ import gettext
 import locale
 import os
 import sys
+
 from codecs import encode, decode
 from subprocess import PIPE
 from subprocess import Popen
 
-version = "2.0.1"
+version = "2.1.0"
 
-log = 0
 win32 = sys.platform.startswith("win")
 
 illegalpathchars = []
@@ -48,8 +48,9 @@ replacementchar = '_'
 
 def CleanFile(filename):
 
-    for char in illegalfilechars:
-        filename = filename.replace(char, replacementchar)
+    if win32:
+        for char in illegalfilechars:
+            filename = filename.replace(char, replacementchar)
 
     return filename
 
@@ -77,6 +78,47 @@ def CleanPath(path, absolute=False):
     return path
 
 
+def get_latest_version():
+
+    import http.client
+    import json
+
+    conn = http.client.HTTPSConnection("api.github.com")
+    conn.request("GET", "/repos/Nicotine-Plus/nicotine-plus/releases/latest", headers={"User-Agent": "Nicotine+"})
+    resp = conn.getresponse()
+    data = json.loads(resp.read().decode("utf-8"))
+
+    hlatest = data['tag_name']
+    latest = int(make_version(hlatest))
+    date = data['created_at']
+
+    return latest, date
+
+
+def make_version(version):
+
+    if version.find("dev") >= 0:
+        # Example: 2.0.1.dev1
+
+        ix = version.find("dev") - 1
+        version = version[:ix]
+    elif version.find("rc") >= 0:
+        # Example: 2.0.1.rc1
+
+        ix = version.find("rc") - 1
+        version = version[:ix]
+
+    s = version.split(".")
+
+    if len(s) >= 4:
+        major, minor, micro, milli = [int(i) for i in s[:4]]
+    else:
+        major, minor, micro = [int(i) for i in s[:3]]
+        milli = 0
+
+    return (major << 24) + (minor << 16) + (micro << 8) + milli
+
+
 def GetUserDirectories():
     """Returns a tuple:
     - the config directory
@@ -113,65 +155,68 @@ def GetResultBitrateLength(filesize, attributes):
     # If there are 3 entries in the attribute list
     if len(attributes) == 3:
 
-        a = attributes
+        first = attributes[0]
+        second = attributes[1]
+        third = attributes[2]
 
         # Sometimes the vbr indicator is in third position
-        if a[2] == 0 or a[2] == 1:
+        if third == 0 or third == 1:
 
-            if a[2] == 1:
+            if third == 1:
                 h_bitrate = " (vbr)"
 
-            bitrate = a[0]
+            bitrate = first
             h_bitrate = str(bitrate) + h_bitrate
 
-            h_length = '%i:%02i' % (a[1] / 60, a[1] % 60)
+            h_length = '%i:%02i' % (second / 60, second % 60)
 
         # Sometimes the vbr indicator is in second position
-        elif a[1] == 0 or a[1] == 1:
+        elif second == 0 or second == 1:
 
-            if a[1] == 1:
+            if second == 1:
                 h_bitrate = " (vbr)"
 
-            bitrate = a[0]
+            bitrate = first
             h_bitrate = str(bitrate) + h_bitrate
 
-            h_length = '%i:%02i' % (a[2] / 60, a[2] % 60)
+            h_length = '%i:%02i' % (third / 60, third % 60)
 
         # Lossless audio, length is in first position
-        elif a[2] > 1:
+        elif third > 1:
 
             # Bitrate = sample rate (Hz) * word length (bits) * channel count
             # Bitrate = 44100 * 16 * 2
-            bitrate = (a[1] * a[2] * 2) / 1000
+            bitrate = (second * third * 2) / 1000
             h_bitrate = str(bitrate)
 
-            h_length = '%i:%02i' % (a[0] / 60, a[0] % 60)
+            h_length = '%i:%02i' % (first / 60, first % 60)
 
         else:
 
-            bitrate = a[0]
+            bitrate = first
             h_bitrate = str(bitrate) + h_bitrate
 
     # If there are 2 entries in the attribute list
     elif len(attributes) == 2:
 
-        a = attributes
+        first = attributes[0]
+        second = attributes[1]
 
         # Sometimes the vbr indicator is in second position
-        if a[1] == 0 or a[1] == 1:
+        if second == 0 or second == 1:
 
             # If it's a vbr file we can't deduce the length
-            if a[1] == 1:
+            if second == 1:
 
                 h_bitrate = " (vbr)"
 
-                bitrate = a[0]
+                bitrate = first
                 h_bitrate = str(bitrate) + h_bitrate
 
             # If it's a constant bitrate we can deduce the length
             else:
 
-                bitrate = a[0]
+                bitrate = first
                 h_bitrate = str(bitrate) + h_bitrate
 
                 # Dividing the file size by the bitrate in Bytes should give us a good enough approximation
@@ -182,10 +227,10 @@ def GetResultBitrateLength(filesize, attributes):
         # Sometimes the bitrate is in first position and the length in second position
         else:
 
-            bitrate = a[0]
+            bitrate = first
             h_bitrate = str(bitrate) + h_bitrate
 
-            h_length = '%i:%02i' % (a[1] / 60, a[1] % 60)
+            h_length = '%i:%02i' % (second / 60, second % 60)
 
     return h_bitrate, bitrate, h_length
 
@@ -198,7 +243,7 @@ def ApplyTranslation():
     If something goes wrong we fall back to no translation.
 
     This function also try to find translation files in the project path first:
-    $(PROJECT_PATH)/languages/$(LANG)/LC_MESSAGES/nicotine.mo
+    $(PROJECT_PATH)/mo/$(LANG)/LC_MESSAGES/nicotine.mo
 
     If no translations are found we fall back to the system path for locates:
     GNU/Linux: /usr/share/locale/$(LANG)/LC_MESSAGES
@@ -211,7 +256,7 @@ def ApplyTranslation():
     PACKAGE = 'nicotine'
 
     # Local path where to find translation (mo) files
-    LOCAL_MO_PATH = 'languages'
+    LOCAL_MO_PATH = 'mo'
 
     # Python 2.7.X is build via Visual Studio 2008 on Windows:
     # https://stackoverflow.com/questions/32037573/load-gtk-glade-translations-in-windows-using-python-pygobject
@@ -247,7 +292,10 @@ def ApplyTranslation():
 
     else:
         # Unix locales handling: We let the system handle the locales
-        locale.setlocale(locale.LC_ALL, '')
+        try:
+            locale.setlocale(locale.LC_ALL, '')
+        except Exception as e:
+            print("Error while attempting to set locale: %s" % e)
 
     # Gettext handling
     if gettext.find(PACKAGE, localedir=LOCAL_MO_PATH) is None:
@@ -262,10 +310,10 @@ def ApplyTranslation():
         # Locales are in the current dir: install them
         if win32:
 
-            # On windows we use intl.dll: the core DLL of GNU gettext-runtime on Windows
+            # On windows we use libintl-8.dll: the core DLL of GNU gettext-runtime on Windows
             import ctypes
 
-            libintl = ctypes.cdll.LoadLibrary("intl.dll")
+            libintl = ctypes.cdll.LoadLibrary("libintl-8.dll")
 
             libintl.bindtextdomain(PACKAGE, LOCAL_MO_PATH)
             libintl.bind_textdomain_codeset(PACKAGE, "UTF-8")
@@ -280,33 +328,6 @@ def ApplyTranslation():
     gettext.textdomain(PACKAGE)
 
 
-def displayTraceback(exception=None):
-
-    global log
-    import traceback
-
-    if exception is None:
-        tb = traceback.format_tb(sys.exc_info()[2])
-    else:
-        tb = traceback.format_tb(exception)
-
-    if log:
-        log("Traceback: " + str(sys.exc_info()[0].__name__) + ": " + str(sys.exc_info()[1]))
-
-    for line in tb:
-        if type(line) is tuple:
-            xline = ""
-            for item in line:
-                xline += str(item) + " "
-            line = xline
-
-        line = line.strip("\n")
-        if log:
-            log(line)
-
-    traceback.print_exc()
-
-
 def unescape(string):
     """Removes quotes from the beginning and end of strings, and unescapes it."""
 
@@ -414,45 +435,6 @@ def executeCommand(command, replacement=None, background=True, returnoutput=Fals
     return procs[-1].communicate()[0]
 
 
-def findBestEncoding(bytes, encodings, fallback=None):
-    """Tries to convert the bytes with the encodings, the first successful conversion is returned.
-
-    If none match the fallback encoding will be used with the 'replace' argument. If no fallback is
-    given the first encoding from the list is used."""
-
-    if isinstance(bytes, str):
-        return bytes
-
-    for encoding in encodings:
-        try:
-            return str(bytes, encoding)
-        except (UnicodeDecodeError, LookupError) as e:  # noqa: F841
-            pass
-
-    # None were successful
-    if fallback:
-        return str(bytes, fallback, 'replace')
-    else:
-        return str(bytes, encodings[0], 'replace')
-
-
-def is_flatpak():
-    """Detect if we are running Flatpak"""
-
-    return sys.platform == "linux" and os.path.exists("/.flatpak-info")
-
-
-def installPrefix():
-    """Returns the system prefix where Nicotine+ is installed"""
-
-    prefix = sys.prefix
-
-    if is_flatpak():
-        prefix = "/app"
-
-    return prefix
-
-
 def strace(function):
     """Decorator for debugging"""
 
